from recipe_scrapers.innit import Innit
from tests import ScraperTest


class TestInnitScraper(ScraperTest):

    scraper_class = Innit

    def test_host(self):
        self.assertEqual("innit.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "Tofu Mixed Greens Salad with Broccoli Beet Mix & Carrot Ginger Dressing",
            self.harvester_class.title(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Innit Inc")

    def test_total_time(self):
        self.assertEqual(51, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Carrots",
                "1 piece Fresh Ginger",
                "1/2 Orange",
                "1 Tbsp Fresh Chives",
                "2 cups Broccoli",
                "1 lb Precooked Beets",
                "2 Tbsp Italian Parsley",
                "4 cups Fresh Spring Mix",
                "2 packages Extra Firm Tofu",
                "1 Tbsp Miso Paste",
                "1/2 tsp Sesame Seed Oil",
                "1/2 Tbsp Honey",
                "1 1/3 tsp Kosher Salt",
                "2/3 cup Olive Oil",
                "2 pinches Black Pepper",
                "1/4 cup Rice Wine Vinegar",
                "1 cup Sunflower Seeds",
            ],
            self.harvester_class.ingredients(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "sugarContent": "18 g",
                "proteinContent": "32 g",
                "fiberContent": "11 g",
                "unsaturatedFatContent": "55 g",
                "fatContent": "64 g",
                "cholesterolContent": "0 mg",
                "calories": "830 kcal",
                "carbohydrateContent": "34 g",
                "saturatedFatContent": "9 g",
                "sodiumContent": "1060 mg",
            },
            self.harvester_class.nutrients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Preheat
Preheat the oven to 425F.
Line sheet pan with foil.
Sear Tofu
Drain, pat dry & prepare tofu.
Heat pan on high heat for 2 minutes.
Cook for 7 min or until golden brown on all sides, seasoning half way.
Remove from pan.
Bake Broccoli
Toss broccoli with oil & salt.
Bake for 22 minutes.
Simmer Carrots & Ginger
Prepare ingredients.
Pre-heat pan. Add all ingredients; cover with water.
Cook until soft, about 5 minutes.
Blend Dressing Ingredients
Transfer carrot-ginger mixture to blender.
Add orange juice/zest, miso, sesame oil, honey, rice vinegar.
Blend until smooth.
Let cool in fridge for 10 minutes. Fold in chives & season.
Flavor Beets
Toss beets in oil. Season with salt, pepper & parsley.
Toast Sunflower Seeds
Combine ingredients.
Toast in oven for 5 - 7 min or until golden brown.
Mixed Greens
Wash greens & dry.
Serve and Enjoy!
Pair with your favorite music!""",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "https://www.innit.com/meal-service/en-US/images/Meal-Salads%3A%20Blended-Carrot_Ginger_Dressing%2BAssembled-Broccoli_Beet_Mix%2BSeared-Tofu-Diced%2BOlive_Oil%2BPrepared-Mixed_Greens_480x480.png",
            self.harvester_class.image(),
        )



from recipe_scrapers.momswithcrockpots import MomsWithCrockPots
from tests import ScraperTest


class TestMomsWithCrockPotsScraper(ScraperTest):

    scraper_class = MomsWithCrockPots

    def test_host(self):
        self.assertEqual("momswithcrockpots.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://momswithcrockpots.com/slow-cooked-macaroni-cheese/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Crockpot Macaroni & Cheese")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Karen McCormick")

    def test_total_time(self):
        self.assertEqual(225, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "8 ounces macaroni",
                "2 teaspoons olive oil",
                "1 cup evaporated milk",
                "1/2 cup milk",
                "1/2 teaspoon salt",
                "1/4 teaspoon ground black pepper",
                "2 cups Cheddar cheese (shredded, or a Cheddar blend)",
                "4 tablespoons butter (melted)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'Cook the macaroni following package directions. Drain in a colander and rinse with hot water. Drain well.\nGenerously butter the sides and bottom of a 3 1/2- to 4-quart slow cooker (I use about 2 tablespoons of butter).\nCombine the macaroni with the remaining ingredients in the slow cooker and blend well. Cover the slow cooker and cook on LOW for 2 1/2 to 3 1/2 hours, stirring a few times.\nIf desired, spoon the cooked macaroni and cheese mixture into a baking dish, sprinkle with a little more cheese, and put under the broiler for a minute or 2, just until cheese is melted.\nWhen the macaroni and cheese is done, feel free to spoon into a baking dish, top with a little more cheese, and put under the broiler for a minute or two for that "fresh from the oven" look.\nUse a gluten free pasta',
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper


class FineDiningLovers(AbstractScraper):
    @classmethod
    def host(cls):
        return "finedininglovers.com"

    def title(self):
        return self.schema.title()

    def author(self):
        container = self.soup.find("div", {"class": "author-name"})
        if container:
            return container.find("a").get_text()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def image(self):
        return self.schema.image()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class BBCFood(AbstractScraper):
    @classmethod
    def host(cls, domain="com"):
        return f"bbc.{domain}"

    def title(self):
        return normalize_string(self.soup.find("h1").get_text())

    def total_time(self):
        return sum(
            [
                get_minutes(
                    self.soup.find("p", {"class": "recipe-metadata__prep-time"})
                ),
                get_minutes(
                    self.soup.find("p", {"class": "recipe-metadata__cook-time"})
                ),
            ]
        )

    def yields(self):
        return get_yields(self.soup.find("p", {"class": "recipe-metadata__serving"}))

    def author(self):
        container = self.soup.find("div", {"class": "chef__name"})
        if not container:
            return None

        author = container.a
        return author.text if author else None

    def image(self):
        container = self.soup.find(True, {"class": "recipe-media__image"})
        if not container:
            return None

        image = container.parent.find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = self.soup.findAll(
            "li", {"class": "recipe-ingredients__list-item"}
        )

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.findAll(
            "p", {"class": "recipe-method__list-item-text"}
        )

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



from recipe_scrapers.bbcfood import BBCFood
from tests import ScraperTest


class TestBBCFoodScraper(ScraperTest):

    scraper_class = BBCFood

    def test_host(self):
        self.assertEqual("bbc.com", self.harvester_class.host())

    def test_host_domain(self):
        self.assertEqual("bbc.co.uk", self.harvester_class.host(domain="co.uk"))

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Rob Burns")

    def test_canonical_url(self):
        self.assertEqual(
            "http://www.bbc.co.uk/food/recipes/baileysandchocolatec_72293",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Irish cream and chocolate cheesecake"
        )

    def test_total_time(self):
        self.assertEqual(130, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 item", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://ichef.bbci.co.uk/food/ic/food_16x9_608/recipes/baileysandchocolatec_72293_16x9.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "100g/3½oz butter",
                "250g/8¾oz digestive biscuits, crushed",
                "600g/1lb 5oz cream cheese",
                "25ml/1fl oz Baileys or other Irish cream liqueur",
                "100ml/3½oz icing sugar",
                "300ml/10½oz double cream, whipped",
                "100g/3½oz grated chocolate",
                "200ml/7¼oz double cream, whipped",
                "cocoa powder, to dust",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Melt the butter in a pan and add the crushed digestive biscuits. Mix well until the biscuits have absorbed all the butter.\nRemove from the heat and press into the bottom of a lined 18cm/7in springform tin. Place in the refrigerator and allow to set for one hour.\nMeanwhile, prepare the filling. Lightly whip the cream cheese then beat in the Irish cream and icing sugar. Fold in the whipped cream and grated chocolate. When smooth, spoon evenly onto the biscuits.\nRefrigerate and allow to set for a further two hours. Once set, remove and decorate with whipped cream and cocoa powder dusted over the top. Serve.",
            self.harvester_class.instructions(),
        )



import responses

from recipe_scrapers.kptncook import KptnCook
from tests import ScraperTest


class TestKptnCookScraper(ScraperTest):

    scraper_class = KptnCook

    @classmethod
    def expected_requests(cls):
        yield responses.GET, "https://mobile.kptncook.com/recipe/pinterest/Low-Carb-Tarte-Flamb%C3%A9e-with-Serrano-Ham-%26-Cream-Cheese/315c3c32?lang=en", "tests/test_data/kptncook.testhtml"
        yield responses.POST, "https://mobile.kptncook.com/recipes/search?kptnkey=6q7QNKy-oIgk-IMuWisJ-jfN7s6&lang=en", "tests/test_data/kptncook.testjson"

    def test_host(self):
        self.assertEqual("mobile.kptncook.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(
            "Laura (http://lauraleanskitchen.com/de/)", self.harvester_class.author()
        )

    def test_title(self):
        self.assertEqual(
            "Low Carb Tarte Flambée with Serrano Ham & Cream Cheese",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual("Pork", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_prep_time(self):
        self.assertEqual(20, self.harvester_class.prep_time())

    def test_cook_time(self):
        self.assertEqual(5, self.harvester_class.cook_time())

    def test_yields(self):
        self.assertEqual(1, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://d2am1qai33sroc.cloudfront.net/image/59b8e8a6950000a90949a083?kptnkey=6q7QNKy-oIgk-IMuWisJ-jfN7s6",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        ingredients = [
            "1.0 egg",
            "0.5 cup(s) cream cheese / curd cheese",
            "salt",
            "pepper",
            "2.0 tbsp cream cheese",
            "2.0 slice(s) serrano ham",
            "0.5 cup(s) arugula",
            "0.5 cup(s) cherry tomato",
            "0.5 tsp herbes de Provence, dried",
            "0.25 tsp chili flake",
        ]
        self.assertEqual(ingredients, self.harvester_class.ingredients())

    def test_instructions(self):
        instructions = "All set?\nPreheat oven to 360°F (conventional, recommended) or 325°F (fan).\nMix curd, eggs, chili flakes, herbes de Provence, salt, and pepper to a bowl and mix.\nPlace the mass on a lined baking sheet, spread thinly (make sure the layer is very thin) and bake for 10-15 min.\nWash and slice cherry tomatoes.\nWash and drain arugula.\nRemove the base from the oven and spread with cream cheese. Top with tomatoes and Serrano ham, and bake for another 5 min.\nRemove the tarte flambée from the oven, top with arugula, and enjoy."
        self.assertEqual(instructions, self.harvester_class.instructions())

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Flourless but so delicious!", self.harvester_class.description()
        )

    def test_language(self):
        self.assertEqual("en", self.harvester_class.language())

    def test_nutrients(self):
        self.assertEqual(
            {"calories": 368, "protein": 34, "fat": 22, "carbohydrate": 9},
            self.harvester_class.nutrients(),
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://mobile.kptncook.com/recipe/pinterest/Low-Carb-Tarte-Flamb%C3%A9e-with-Serrano-Ham-%26-Cream-Cheese/315c3c32?lang=en",
            self.harvester_class.canonical_url(),
        )



from recipe_scrapers.allrecipes import AllRecipesCurated, AllRecipesUser
from tests import ScraperTest


class TestAllRecipesCuratedScraper(ScraperTest):

    scraper_class = AllRecipesCurated

    def test_host(self):
        self.assertEqual("allrecipes.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Michelle", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.allrecipes.com/recipe/133948/four-cheese-margherita-pizza/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Four Cheese Margherita Pizza")

    def test_description(self):
        self.assertEqual(
            self.harvester_class.description(),
            "This is a fantastic version of an Italian classic. The feta cheese adds a rich flavor that brings this dish to life. Incredibly easy and incredibly delicious!",
        )

    def test_cook_time(self):
        self.assertEqual(10, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(15, self.harvester_class.prep_time())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fimages.media-allrecipes.com%2Fuserphotos%2F694708.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "¼ cup olive oil",
                "1 tablespoon minced garlic",
                "½ teaspoon sea salt",
                "8 Roma tomatoes, sliced",
                "2 (12 inch) pre-baked pizza crusts",
                "8 ounces shredded Mozzarella cheese",
                "4 ounces shredded Fontina cheese",
                "10 fresh basil leaves, washed, dried",
                "½ cup freshly grated Parmesan cheese",
                "½ cup crumbled feta cheese",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Stir together olive oil, garlic, and salt; toss with tomatoes, and allow to stand for 15 minutes. Preheat oven to 400 degrees F (200 degrees C).\nBrush each pizza crust with some of the tomato marinade. Sprinkle the pizzas evenly with Mozzarella and Fontina cheeses. Arrange tomatoes overtop, then sprinkle with shredded basil, Parmesan, and feta cheese.\nBake in preheated oven until the cheese is bubbly and golden brown, about 10 minutes.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.8, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("", self.harvester_class.cuisine())

    def test_category(self):
        self.assertEqual(
            "World Cuisine Recipes,European,Italian", self.harvester_class.category()
        )


class TestAllRecipesUserScraper(ScraperTest):

    scraper_class = AllRecipesUser

    def test_host(self):
        self.assertEqual("allrecipes.com/cook", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("AOSWALT", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.allrecipes.com/cook/1052065/recipe/78989010-fddc-3e4a-b54f-10e3ad110564",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Amy's Fabulous Meat Rub")

    def test_total_time(self):
        self.assertEqual(5, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.allrecipes.com/img/misc/og-default.png",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 tablespoons white sugar",
                "1 tablespoon brown sugar",
                "1 1/2 tsp garlic powder",
                "1 1/2 tsp chili powder",
                "1 1/2 tsp paprika",
                "1 1/2 tsp ground cumin",
                "1 tsp salt",
                "1 tsp onion powder",
                "1 1/2 tsp ground black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a small bowl, combine all ingredients. Rub down your choice of meat, and let marinate for at least an hour.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        return self.assertIsNone(self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())



from recipe_scrapers.indianhealthyrecipes import IndianHealthyRecipes
from tests import ScraperTest


class TestIndianHealthyRecipesScraper(ScraperTest):

    scraper_class = IndianHealthyRecipes

    def test_host(self):
        self.assertEqual("indianhealthyrecipes.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Swasthi", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Banana Cake Recipe", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(50, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("16 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.indianhealthyrecipes.com/wp-content/uploads/2021/01/banana-cake-recipe.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups wheat flour (or all-purpose (refer notes))",
                "2 ½ teaspoons baking powder",
                "½ teaspoon salt ((or ⅓ teaspoon table salt))",
                "1 cup fine sugar ((prefer organic))",
                "100 grams unsalted butter ((soft & cold) (around ½ cup))",
                "2 eggs",
                "2 teaspoons vanilla extract",
                "160 ml milk",
                "1 cup ripe banana (mashed)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Preparation\nBring all the ingredients to room temperature except butter before you begin to prepare the batter.\nButter has to be cold, yet soft and should hold its structure. To check press the block of cold butter with your finger, it should dent slightly yet hold its solid structure and should not be too soft.\nGrease a 8 by 8 inch cake tray and line with parchment paper. If you do not have a parchment paper, you may sprinkle flour all over the tray including the sides. Invert and dust off the excess in your kitchen sink.\nPreheat the oven to 170 C or 340 F for at least 15 minutes. If you have a fan forced oven then preheat to 160 C or 320 F.\nFluff up the flour in the jar/ pack with a fork. Then spoon it to the measuring cup and level it with a knife or a straight edged spoon. Sieve flour, baking powder and salt. Set aside.\nHow To Make Banana Cake\nMake sure butter is soft but still cold before this step. Add butter and sugar to a mixing bowl. Using a whisk, beat together until light, pale & fluffy.\nPour vanilla extract and add 1 egg at a time and beat just until creamy.\nAdd the other egg and beat again just until creamy.\nNext add the sieved flour, salt and baking powder. Mix it gently.\nAdding milk in 2 batches, mix the flour on a medium speed until smooth. Do not over mix it.\nAdd banana puree and mix until just combined. Avoid over mixing.\nPour the batter to a lined cake tray and knock it against the counter a few times. Bake for 25 to 30 mins if using a 8 by 8 square pan. If using a different size pan, then the timing varies.\nA skewer/ tester inserted in the center of the cake comes out clean when the cake is done.\nPlace the cake pan on a wired rack and cool for 10 minutes. Then invert the banana cake on the wire rack.\nCool completely before slicing. Serve banana cake plain with milk or tea.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.95, self.harvester_class.ratings())



from recipe_scrapers.afghankitchenrecipes import AfghanKitchenRecipes
from tests import ScraperTest


class TestafghankitchenrecipesScraper(ScraperTest):

    scraper_class = AfghanKitchenRecipes

    def test_host(self):
        self.assertEqual("afghankitchenrecipes.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("nash", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Mantu (Beef Dumplings)", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(75, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "http://www.afghankitchenrecipes.com/wp-content/uploads/2014/04/manto4-250x212.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups of Onions",
                "1 pound of Ground Beef/Lamb",
                "½ tsp. Garlic",
                "2 tsp. Coriander Powder",
                "½ tsp. Salt",
                "½ tsp. Black Pepper Powder (freshly ground)",
                "1 cup Chana Lentils",
                "½ pound Ground Beef/Lamb",
                "1 medium Onion (finely chopped)",
                "½ tsp. Garlic",
                "½ tsp. Salt",
                "½ tsp. Black Pepper Powder (freshly ground)",
                "2 tsp. Tomato Paste",
                "30 ml. Cooking Oil",
                "1 tsp. Dried Mint Leaves",
                "½ tsp. Garlic",
                "½ tsp. Salt",
                "500 ml. Plain Yogurt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Heat pan over a medium heat and add garlic, ground beef and salt and coriander. Cook until the meat is half-way cooked for about 15 to 20 minutes.",
                    "Add black pepper and remove from heat.",
                    "Combine one cup of raw chopped onions to the meat and let it cool down to room temperature",
                    "Soak split peas for 2–3 hours or overnight.",
                    "Heat oil in a frying pan and cook onion and garlic until lightly browned.",
                    "Add tomato and tomato paste. Add split peas and ¾ cup water and cook for 30–45 minutes until soft.",
                    "Season with salt and pepper and set aside.",
                    "Combine all of the ingredients for dip in a bowl and mix well.",
                    "Place flour in a large mixing bowl and gradually add water, mixing with hands until it becomes doughy.",
                    "Leave the dough to settle for 15–20 minutes or until it becomes firm.",
                    "Separate dough into small handfuls and roll into individual ball shapes.",
                    "Scatter some flour on the bench surface and using a small rolling pin, roll the balls into circular shapes.",
                    "Roll the dough ball into a very thin (1/16-inch) strips using a pasta machine. Cut the strips into 2-inch squares.",
                    "Place approximately one tablespoon of the cooled ground beef and onion mixture onto each wrap. To make the wraps stick together easily, wet the edges with water (you may use your fingers or a basting brush).",
                    "Fold over first two opposite ends of the egg roll wrap and followed by other two ends to enclose dumplings. Press the edges tightly to seal together. Continue with the remaining wraps.",
                    "Take the racks out of the steamer dish. Add water to the dish, cover and bring to boil.",
                    "Oil the base of steamer to prevent sticking and place dumplings carefully across oil.",
                    "Cover lid and cook for approximately 40 minutes.",
                    "When the dumplings have cooked, add a thin layer of the yogurt mixture to a large serving plate. Place the steamed dumplings on top.",
                    "Pour some more of the yogurt mixture on top of the dumplings and coat everything with the the topping sauce.",
                    "Garnish with dried mint and a little bit of cayenne pepper.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.5, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_minutes, normalize_string

INSTRUCTIONS_NUMBERING_REGEX = re.compile(r"^\d{1,2}\.\s*")  # noqa


class Panelinha(AbstractScraper):
    @classmethod
    def host(cls):
        return "panelinha.com.br"

    def title(self):
        return normalize_string(self.soup.find("h1").get_text())

    def total_time(self):
        return get_minutes(
            self.soup.find("span", string="Tempo de preparo").nextSibling
        )

    def ingredients(self):
        ingredients = self.soup.find("h4", string="Ingredientes").nextSibling.findAll(
            "li"
        )

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find(
            "h4", string="Modo de preparo"
        ).nextSibling.findAll("li")

        instructions = [
            normalize_string(instruction.get_text()) for instruction in instructions
        ]

        # Some recipes pages have a different html structure.
        if not instructions:
            instructions = self.soup.find(
                "h4", string="Modo de preparo"
            ).nextSibling.p.strings

            instructions = (
                normalize_string(instruction) for instruction in instructions
            )

            instructions = map(
                lambda step: INSTRUCTIONS_NUMBERING_REGEX.sub("", step), instructions
            )

        return "\n".join(instructions)

    def yields(self):
        return self.schema.yields()



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_minutes


class CookingCircle(AbstractScraper):
    @classmethod
    def host(cls):
        return "cookingcircle.com"

    def author(self):
        return (
            self.soup.find("div", {"class": "recipe-author"})
            .findChild("span", {"class": "text-uppercase"})
            .get_text()
        )

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        ul = self.soup.find("ul", {"class": "single-method-overview__times"})
        total_time = None
        for li in ul.find_all("li"):
            if li.span.get_text().lower() == "total time:":
                total_time = li.span.find_next().get_text()

        if total_time is not None:
            total_time = re.findall("[0-9]+", total_time)[0]
        return get_minutes(total_time)

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ul_list = (
            self.soup.find(
                "div", {"class": "single-ingredients__group", "data-unit": "metric"}
            )
            .findChild("ul", {"class": "single-ingredients__list"})
            .findChildren("li")
        )

        ingredients = []
        for li in ul_list:
            ingredients.append(
                li.get_text().replace("\t", "").replace("\n\n", " ").replace("\n", "")
            )
        return ingredients

    def instructions(self):
        ul_list = self.soup.find("ul", {"class": "single-method__method"}).findChildren(
            "li"
        )

        instructions = []
        for li in ul_list:
            instructions.append(li.get_text().strip().replace("\n", " "))

        return "\n".join(instructions)



from recipe_scrapers.skinnytaste import SkinnyTaste
from tests import ScraperTest


class TestSkinnyTasteScraper(ScraperTest):

    scraper_class = SkinnyTaste

    def test_host(self):
        self.assertEqual("skinnytaste.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.skinnytaste.com/grilled-chicken-with-spinach-and-melted/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Grilled Chicken with Spinach and Melted Mozzarella",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Gina")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.skinnytaste.com/wp-content/uploads/2011/03/Grilled-Chicken-with-Spinach-and-Melted-Mozzarella-10.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "24 oz 3 large chicken breasts sliced in half lengthwise to make 6",
                "kosher salt and pepper to taste",
                "1 tsp olive oil",
                "3 cloves garlic (crushed)",
                "10 oz frozen spinach (drained)",
                "3 oz shredded part skim mozzarella",
                "1/2 cup roasted red pepper (sliced in strips (packed in water))",
                "olive oil spray",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Preheat oven to 400°F. Season chicken with salt and pepper. Lightly spray a grill or grill pan with oil. Cook chicken until no longer pink, about 2 to 3 minutes per side.
Heat a skillet over medium heat. Add oil and garlic, sauté a 30 seconds, add spinach, salt and pepper. Cook until heated through, 2 to 3 minutes.
Place chicken on a baking sheet, divide spinach evenly between the 6 pieces and place on top. Top each with 1/2 oz mozzarella, roasted peppers and bake until melted, about 3 minutes.""",
            self.harvester_class.instructions(),
        )

    def test_total_time(self):
        self.assertEqual(
            17,
            self.harvester_class.total_time(),
        )

    def test_ratings(self):
        self.assertEqual(
            5,
            self.harvester_class.ratings(),
        )



from recipe_scrapers.latelierderoxane import LAtelierDeRoxane
from tests import ScraperTest


class TestLAtelierDeRoxaneScraper(ScraperTest):

    scraper_class = LAtelierDeRoxane

    def test_host(self):
        self.assertEqual("latelierderoxane.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.latelierderoxane.com/blog/recette-cake-marbre/",
            self.harvester_class.canonical_url(),
        )

    def test_image(self):
        self.assertEqual(
            "https://www.latelierderoxane.com/blog/wp-content/uploads/cake-marbre..png",
            self.harvester_class.image(),
        )

    def test_title(self):
        self.assertEqual(
            "Recette cake marbré au chocolat facile", self.harvester_class.title()
        )

    def test_description(self):
        self.assertEqual(
            "Je pense que nous avons déjà tous acheté, au moins une fois, un cake Savane au supermarché ! Un délicieux marbré, moelleux à souhait au bon goût de vanille et cacao. Aujourd’hui, je te propose une recette facile et rapide pour réaliser ce fameux cake marbré maison ! Réalise cette recette et tu n’achèteras plus l’industriel 😉",
            self.harvester_class.description(),
        )

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_prep_time(self):
        self.assertEqual(15, self.harvester_class.prep_time())

    def test_cook_time(self):
        self.assertEqual(45, self.harvester_class.cook_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 œufs",
                "70 g de sucre",
                "70 g de beurre fondu",
                "1 sachet de levure chimique",
                "250 g de farine",
                "150 g de lait",
                "150 g de chocolat noir fondu",
                "1 càc d'arôme ou poudre de vanille",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Préchauffe le four à 165°.\nCommence par fouetter les œufs et le sucre, à l’aide de ton robot ou batteur électrique, pendant 5 minutes : ton mélange doit s’éclaircir et double de volume !\nAjoute le beurre fondu, la levure, la farine et fouette brièvement.\nVerse le lait et fouette jusqu’à l’obtention d’un mélange homogène.\nSépare la préparation obtenue dans deux bols différents.\nDans un des deux bols, ajoute l’arôme ou la poudre de vanille.\nFais fondre ton chocolat, au bain-marie ou au micro-onde, et incorpore-le à l’aide d’une maryse dans ton second bol.\nRécupère un moule à cake et beurre-le.\nVerse, dans le fond du moule, la moitié de la pâte à la vanille puis la moitié de celle au chocolat.\nRenouvèle la même opération une deuxième fois.\nEnfourne pendant 45 min.\nTu peux vérifier la cuisson à l’aide d’un couteau, plante-le au centre de ton cake : ta lame doit ressortir sèche.\nLaisse tiédir ton cake afin de faciliter son démoulage.",
            self.harvester_class.instructions(),
        )

    def test_site_name(self):
        self.assertEqual("L'Atelier de Roxane", self.harvester_class.site_name())



# mypy: disallow_untyped_defs=False
from abc import ABC, abstractmethod
from typing import Iterable


class PluginInterface(ABC):
    """
    Interface that all "Plugins" (including the ones written by programmers
    using the package) should implement.

    Every plugin should have the following 2 methods implemented:

    - should_run
    - run
    """

    run_on_hosts: Iterable[str] = ("*",)
    run_on_methods: Iterable[str] = ("title",)

    @classmethod
    @abstractmethod
    def run(cls, decorated):
        pass

    @classmethod
    def should_run(cls, host, method):
        return cls._should_run_host_check(host) and cls._should_run_method_check(method)

    @classmethod
    def _should_run_host_check(cls, host):
        return "*" in cls.run_on_hosts or host in cls.run_on_hosts

    @classmethod
    def _should_run_method_check(cls, method):
        return method in cls.run_on_methods



from recipe_scrapers.franzoesischkochen import FranzoesischKochen
from tests import ScraperTest


class TestFranzoesischKochenScraper(ScraperTest):

    scraper_class = FranzoesischKochen

    def test_host(self):
        self.assertEqual("franzoesischkochen.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Aurélie Bastian", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Tourte mit Feigen und Confit de canard", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(90, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual(None, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.franzoesischkochen.de/wp-content/uploads/2020/09/Tourte-confit-de-canard.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4-5 Feigen",
                "1 Confit de Canard (Brust bei mir oder 2 Keulen)",
                "3 Schalotten",
                "200-250 g Pfifferlingen",
                "1 Nelke",
                "1 Lorbeerblatt",
                "3 Kardamomkapseln",
                "1 TL Kastanien Honig",
                "150 ml Rotwein (optional)",
                "Rosmarin",
                "Thymian",
                "Salz und Pfeffer",
                "1 Rolle Blätterteig (Quadratisch",
                "oder 2 runde)",
                "50 g gehobelte Mandeln",
                "Milch zum Pinseln.",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "1- Die Schalotten schälen und in dünnen Scheiben schneiden....\n2- In einer großen Pfanne 2-3 EL Entenfett (aus den Confit Dose) geben ... und erhitzen... Die Schalotten darin braten und dann das Fleisch geben. 5 Minuten anbraten\n3- In einer kleinen Pfanne die gehobelten Mandeln kurz anbraten. Auf die Seite stellen.\n4- Die Feigen in kleinen Scheiben schneiden und in einer Pfanne mit ein bisschen Rotwein (oder Wasser) und dem Kardamom köcheln lassen... dazu noch 1 gute TL Kastanien Honig geben.\n5- Den Blätterteig ausrollen, halbieren und in einen 20 cm Ring (oder eine Backform legen). Die Fleisch Pfanne eingeben (das Lorbeerblatt und die Nelken herausfischen) ... Schön plattdrücken, so dass alles passt (wir hatten 1 EL zu viel ... leider leider direkt beim Zubereiten vernascht! )",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.thekitchn import TheKitchn
from tests import ScraperTest


class TestKitchnScraper(ScraperTest):

    scraper_class = TheKitchn

    def test_host(self):
        self.assertEqual("thekitchn.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thekitchn.com/manicotti-22949270",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "How To Make the Best Beef and Cheese Manicotti",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Meghan Splawn")

    def test_total_time(self):
        self.assertEqual(65, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.apartmenttherapy.info/image/upload/f_jpg,q_auto:eco,c_fill,g_auto,w_1500,ar_16:9/k%2FPhoto%2FRecipes%2F2019-10-how-to-beef-manicotti%2F2019-10-04_Kitchn86818_HT-Beef-Manicotti",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 tablespoons olive oil",
                "12 ounces lean ground beef",
                "1 large shallot, finely chopped",
                "4 cloves garlic, minced",
                "3 cups marinara sauce, divided",
                "Cooking spray",
                "14 dried manicotti pasta tubes (8 ounces)",
                "1 (15 to 16-ounce) container full-fat ricotta cheese",
                "2 cups shredded part-skim mozzarella cheese, divided",
                "1/2 cup finely grated Parmesan cheese, divided",
                "1/2 cup chopped fresh parsley leaves, divided",
                "1 large egg, lightly beaten",
                "1 teaspoon kosher salt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Step 1\nCook the beef for the filling. Heat the oil in a large skillet over medium-high heat until shimmering. Add the ground beef and cook until the meat begins to brown, about 5 minutes. Add the shallot and garlic, cook until the shallot is translucent and the beef is cooked through, about 5 minutes more. Transfer to a large bowl and set aside to cool to room temperature.\nStep 2\nHeat the oven to 375ºF and prepare the baking dish. Arrange a rack in the middle of the the oven and heat the oven to 375ºF. Meanwhile, bring a large pot of heavily salted water to a boil. Coat a 9x13-inch baking dish with cooking spray. Spread about 1 1/2 cups of the marinara sauce in the bottom of the dish and set aside.\nStep 3\nBoil the manicotti shells. Add the manicotti to the water and boil until they are al dente, about 8 minutes. Drain and set aside.\nStep 4\nMix up the filling. Add the ricotta, half of mozzarella, half of the Parmesan, half of the parsley, the egg, and the salt to the cooled beef mixture and stir to combine well.\nStep 5\nFill the shells. Transfer the beef and cheese mixture to a piping bag or gallon size ziptop bag. Snip off a 1/2-inch hole in one bottom corner of the bag, then pipe the filling into each manicotti tube (about generous 1/3 cup each). Nestle each filled manicotti in the sauce, packed them tightly together in a single layer.\nStep 6\nTop the filled shells with the remaining sauce and cheeses. Pour the remaining 1 1/2 cups sauce evenly over the manicotti and sprinkle with the remaining mozzarella and Parmesan.\nStep 7\nBake for 35 to 40 minutes. Bake uncovered until the sauce is bubbly, the cheese is browned, and the noodles are very tender, 35 to 40. Let cool for about 10 minutes before serving. Sprinkle with the remaining parsley just before serving.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper


class RutgerBakt(AbstractScraper):
    @classmethod
    def host(cls):
        return "rutgerbakt.nl"

    def author(self):
        return "Rutger van den Broek"

    def title(self):
        title_raw = self.schema.title()
        return title_raw.replace(" – recept", "").replace(" – Recept", "")

    def category(self):
        category = (
            self.url.split("rutgerbakt.nl/")[-1]
            .split("/")[0]
            .replace("-recepten", "")
            .replace("-", " ")
        )
        return category

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        # The yields are all over the place. There is no way to parse this.
        return None

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        # Find the "instructions" heading. It is not really clear when that is.
        # So several steps need to be taken.

        # 1. Filter all the headings.
        # Some h2 headers contain photos. Those are not what we want.
        def filter_heading(headings_old):
            headings_new = []
            for _heading in headings_old:
                if "class" in _heading.attrs:
                    h_class = _heading.attrs["class"]
                    if any("photo" in attr.lower() for attr in h_class):
                        continue
                headings_new.append(_heading)
            return headings_new

        headings = filter_heading(self.soup.find_all("h2"))

        # 2. Find the instructions heading
        # Usually the last heading are the instructions
        # (therefore headings.reverse()).
        # But it's double checked whether the heading contains any of
        # the keywords.
        # I split the heading into words so I keep word boundaries in check.
        headings.reverse()
        for heading in headings:
            tokens = heading.getText().lower().split(" ")
            keywords = ["recept", "bereiding", "maken", "maak"]
            if any(keyword in tokens for keyword in keywords):
                break

        # This function iterates over every next element after the heading.
        def parse_instructions(element):
            for instruction in element.next_siblings:
                if instruction.name not in ["p", "h2", "h3", "h4"]:
                    continue
                if any(
                    item in instruction.text.lower()
                    for item in ["foto: ", "foto's: ", "foto’s: "]
                ):
                    continue
                yield instruction.text.replace("\n", " ").strip()

        instructions = parse_instructions(heading)
        return "\n".join(instructions)

    def ratings(self):
        return self.schema.ratings()

    def description(self):
        # Assuming the first paragraph is the description.
        return self.soup.find("p").text.replace("\n", " ")



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class StreetKitchen(AbstractScraper):
    @classmethod
    def host(cls):
        return "streetkitchen.hu"

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).get_text()

    def total_time(self):
        return None

    def image(self):
        return (
            self.soup.find("div", {"class": "article-featured-image-bg"})
            .find("noscript")
            .find("img")["src"]
        )

    def ingredients(self):
        ingredients = []
        ingredient_group = self.soup.find("div", {"class": "ingredient-group"}).findAll(
            "dd"
        )

        for ingredient in ingredient_group:
            ingredients.append(normalize_string(ingredient.get_text()).strip())

        return ingredients

    def instructions(self):
        instructions = self.soup.find("div", {"class": "the-content-div"}).findAll("p")[
            :-1
        ]  # the last paragraph is advertisement, not instructions
        instructions_arr = []
        for instruction in instructions:
            instructions_arr.append(instruction.get_text())
        return "\n".join(instructions_arr)

    def yields(self):
        return get_yields(
            self.soup.find("span", {"class": "quantity-number"}).get_text()
        )



from recipe_scrapers.heb import HEB
from tests import ScraperTest


class TestHEBScraper(ScraperTest):

    scraper_class = HEB

    def test_host(self):
        self.assertEqual("heb.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.heb.com/recipe/recipe-item/truffled-spaghetti-squash/1398755977632",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Truffled Spaghetti Squash", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "H-E-B")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 large spaghetti squash",
                "1 Tsp Sabatino Truffle salt",
                "4 Tbsp unsalted butter",
                "1 Tbsp Rustico Truffle Oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Preheat oven to 400˚F. Cut spaghetti squash in half from top to bottom (lengthwise).",
                    "Place the spaghetti squash halves, cut side down onto a sheet pan. Roast for 30-45 minutes or until a knife can pierce the outside of skin easily, like a baked potato.",
                    "Once Squash is fully cooked and soft remove it from oven and let it cool for 10 minutes before scooping out meat of squash. In a serving bowl scoop or with a fork scrape out squash and add truffle salt, butter and truffle oil.",
                    "Toss all ingredients together until butter is fully melted. Season to taste if needed and serve warm.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "https://images.heb.com/is/image/HEBGrocery/rcp-thumbnail/spicy-spaghetti-squash-boats-recipe.jpg",
            self.harvester_class.image(),
        )



import unittest

from recipe_scrapers._exceptions import SchemaOrgException
from recipe_scrapers._schemaorg import SchemaOrg


class TestSchemaOrg(unittest.TestCase):
    def setUp(self):
        with open("tests/test_data/schemaorg.testhtml", encoding="utf-8") as pagedata:
            self.schema = SchemaOrg(pagedata.read())

    def test_total_time_with_schema_missing_all_data_should_raise_exception(self):
        keys = ["totalTime", "cookTime", "prepTime"]
        for k in keys:
            if k in self.schema.data:
                del self.schema.data[k]
        with self.assertRaises(SchemaOrgException):
            self.assertEqual(self.schema.total_time(), None)

    def test_total_time_with_schema__all_zeros(self):
        keys = ["totalTime", "cookTime", "prepTime"]
        for k in keys:
            self.schema.data[k] = "PT0M"
        self.assertEqual(self.schema.total_time(), 0)
        del self.schema.data["totalTime"]
        self.assertEqual(self.schema.total_time(), 0)

    def test_nutrient_retrieval(self):
        expected_nutrients = {
            "calories": "240 calories",
            "fatContent": "9 grams fat",
        }
        self.assertEqual(self.schema.nutrients(), expected_nutrients)

    def test_graph_schema_without_context(self):
        with open(
            "tests/test_data/schemaorg_graph.testhtml", encoding="utf-8"
        ) as pagedata:
            schema = SchemaOrg(pagedata.read())
        self.assertNotEqual(schema.data, {})



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import normalize_string


class AlbertHeijn(AbstractScraper):
    @classmethod
    def host(cls):
        return "ah.nl"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        # try schema.org
        instructions = self.schema.instructions()
        if instructions:
            return instructions

        instructions = [
            step
            # get steps root
            for root in self.soup.findAll(
                "div",
                {"class", re.compile("recipe-preparation-steps_root.*")},
            )
            # get steps
            for step in root.findAll("p")
        ]

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



# mypy: disallow_untyped_defs=False
import json
import re

import requests

from ._abstract import HEADERS, AbstractScraper
from ._exceptions import ElementNotFoundInHtml
from ._utils import normalize_string

ID_PATTERN = re.compile(r"/(\d+)-")
SCRIPT_PATTERN = re.compile(
    'gon\\.current_brand="(?P<brand>[^"]+?)".*?'
    'gon\\.current_country="(?P<country>[^"]+?)".*?'
    'gon\\.api_token="(?P<token>[^"]+?)".*?'
    'gon\\.api_host="(?P<host>[^"]+?)".*?'
)

PREPARATION_DICT = {  # these values I was able to retrieve from website
    "time_level_1": 10,  # on the website they are displayed like `5-10 minutes`, I used avg or similar rounded value
    "time_level_2": 15,
    "time_level_3": 20,
    "time_level_4": 25,
    "time_level_5": 35,
}


class MarleySpoon(AbstractScraper):
    def __init__(self, url, proxies=None, timeout=None, *args, **kwargs):
        super().__init__(url=url, proxies=proxies, timeout=timeout, *args, **kwargs)

        # The website's html does not contain any recipe data, but it loads it with a json request.
        # We read the request parameters from html and preform additional request it to fetch recipe data.
        api_url, api_token = self._get_json_params()
        self.page_data = requests.get(
            api_url,
            headers={"authorization": api_token, **HEADERS},
            proxies=proxies,
            timeout=timeout,
        ).content

        self.data = json.loads(self.page_data)

    def _get_json_params(self):
        recipe_id = None
        api_url = None
        api_token = None

        match = ID_PATTERN.search(self.canonical_url())
        if match:
            recipe_id = match.group(1)

        scripts = self.soup.find_all("script")
        for script in scripts:
            matches = SCRIPT_PATTERN.search(str(script.string))
            if matches:
                data = matches.groupdict()
                host = data["host"].replace("\\", "")
                api_url = f"{host}/recipes/{recipe_id}?brand={data['brand']}&country={data['country']}&product_type=web"
                api_token = f"Bearer {data['token']}"

        if recipe_id is None:
            raise ElementNotFoundInHtml("Recipe ID is unknown.")

        if api_url is None or api_token is None:
            raise ElementNotFoundInHtml("Required script not found.")

        return api_url, api_token

    @classmethod
    def host(cls, domain="com"):
        return f"marleyspoon.{domain}"

    def title(self):
        return self.data.get("name_with_subtitle")

    def total_time(self):
        return PREPARATION_DICT.get(self.data.get("preparation_time"), 60)

    def yields(self):
        # The backend of MarleySpoon always returns ingredients for 2 servings
        # This conclusion is made based on personal observations and available plans https://marleyspoon.com/select-plan
        return "2 servings"

    def image(self):
        return self.data.get("image").get("large")

    def nutrients(self):
        return self.data.get("nutrition")

    def ingredients(self):
        ingredients = [
            normalize_string(ingredient.get("name"))
            for ingredient in self.data.get("ingredients")
        ]
        assumed = [
            normalize_string(ingredient.get("name"))
            for ingredient in self.data.get("assumed_ingredients")
        ]
        return ingredients + assumed

    def instructions(self):
        return "\n".join(
            [
                normalize_string(instruction.get("description").replace("__", ""))
                for instruction in self.data.get("steps")
            ]
        )

    def author(self):
        return self.data.get("chef").get("name")

    def description(self):
        return self.data.get("description")

    def links(self):
        links = super().links()
        # this is a useful link to a print card, maybe someone needs it
        links.append({"href": self.data.get("recipe_card_url")})
        return links

    def language(self):
        try:
            # in normal scenario, there will be html `lang` tag and language can be retrieved from it
            return super().language()
        except AttributeError:
            # but during the test, we load json as main resource, but using the `country` property, we can guess it
            return self.data.get("country")



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields


class Bodybuilding(AbstractScraper):
    @classmethod
    def host(cls):
        return "bodybuilding.com"

    def author(self):
        return self.soup.find("span", {"class": "bb-recipe-author-name"}).get_text()

    def title(self):
        return self.soup.find("h1", {"class": "bb-recipe-headline-title"}).get_text()

    def category(self):
        tag_list = self.soup.find(
            "div", {"class": "bb-recipe__desktop-tags"}
        ).findChildren("div", {"class": "bb-recipe__topic"})

        categories = []
        for tag in tag_list:
            categories.append(tag.get_text())
        return ",".join(categories)

    def total_time(self):
        return get_minutes(
            self.soup.find("div", {"class": "bb-recipe__meta-time"})
            .find("div", {"class": "bb-recipe__meta-value-text"})
            .get_text()
        )

    def yields(self):
        return get_yields(
            self.soup.find("div", {"class": "bb-recipe__meta-servings"})
            .find("div", {"class": "bb-recipe__meta-value-text"})
            .get_text()
        )

    def image(self):
        container = self.soup.find("div", {"class": "bb-recipe-header-image"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ul_list = self.soup.find(
            "ul", {"class": "bb-recipe__ingredient-list"}
        ).findChildren("li")

        ingredients = []
        for li in ul_list:
            ingredients.append(" ".join(li.get_text().split()))
        return ingredients

    def instructions(self):
        ol_list = self.soup.find(
            "ol", {"class": "bb-recipe__directions-list"}
        ).findChildren("li")

        count = 0
        instructions = []

        for li in ol_list:
            count += 1
            instructions.append(str(count) + ". " + " ".join(li.get_text().split()))

        return "\n".join(instructions)

    def description(self):
        return (
            self.soup.find("p", {"class": "BBCMS__content--article-description"})
            .get_text()
            .strip()
        )



from recipe_scrapers.g750g import G750g
from tests import ScraperTest


class TestG750gScraper(ScraperTest):

    scraper_class = G750g

    def test_host(self):
        self.assertEqual("750g.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Salade de carottes cuites et crues à l'orange",
        )

    def test_author(self):
        self.assertEqual("750g La Table", self.harvester_class.author())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://static.750g.com/images/600-600/72dcbbdc4c59e14ea2e93632904fc482/salade-de-carottes-cuites-et-crues-a-l-orange.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500g de carottes",
                "3 jus d'orange",
                "1 c.c de coriandre en grain",
                "1 branche de thym",
                "30g d'huile d'olive",
                "2 oranges",
                "1 fenouil",
                "10 cl de vinaigrette citron",
                "500 g de carottes",
                "1/4 de botte de coriandre hachée",
                "Sel",
                "Piment d'Espelette",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Étape 1 : Cuisson des carottes\nDécoupez les carottes en rondelles et faites-les revenir à feu moyen à l'huile d'olive avec les graines de coriandre grossièrement concassées.
\nDéglacez au jus d'orange.
\nAjoutez la branche de thym ainsi que l'assaisonnement.
\nLaissez cuire avec un couvercle une dizaine de minutes puis laissez refroidir. Étape 2 : Préparation de la salade\nRâpez les carottes.
\nDans un saladier, mélangez les carottes râpées, les carottes cuites, le fenouil et les oranges en cubes.
\nAjoutez la vinaigrette et mélangez.
\nAjoutez la coriandre ciselée et le piment d'espelette.""",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.tineno import TineNo
from tests import ScraperTest


class TestTineNoScraper(ScraperTest):
    scraper_class = TineNo

    def test_host(self):
        self.assertEqual("tine.no", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.tine.no/oppskrifter/middag-og-hovedretter/kylling-og-fjarkre/rask-kylling-tikka-masala",
            self.harvester_class.canonical_url(),
        )

    def test_author(self):
        self.assertEqual("TINE", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Rask kylling tikka masala", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("middag", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.tine.no/_/recipeimage/w_2880%2Ch_1620%2Cc_fill%2Cx_764%2Cy_430%2Cg_xy_center/recipeimage/yshftxnhdmojzhelrupo.png",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 dl basmatiris",
                "400 g kyllingfileter",
                "1 ss TINE® Meierismør til steking",
                "1 stk paprika",
                "0.5 dl chili",
                "3 stk vårløk",
                "1 ts hvitløksfedd",
                "1 ss hakket, frisk ingefær",
                "0.5 dl hakket, frisk koriander",
                "2 ts garam masala",
                "3 dl TINE® Lett Crème Fraîche 18 %",
                "3 ss tomatpuré",
                "0.5 ts salt",
                "0.25 ts pepper",
                "0.5 dl slangeagurk",
                "3 dl TINE® Yoghurt Naturell",
                "0.5 dl frisk mynte",
                "1 ts hvitløksfedd",
                "0.5 ts salt",
                "0.25 ts pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Kok ris etter anvisningen på pakken.\nTikka masala:\nDel kylling i biter. Brun kyllingen i smør i en stekepanne på middels varme.\nRens og hakk paprika, chili, vårløk og hvitløk og ha det i stekepannen sammen med kyllingen. Rens og finhakk ingefær og frisk koriander. Krydre med garam masala, koriander og ingefær.\nHell i crème fraîche og tomatpuré, og la småkoke i 5 minutter. Smak til med salt og pepper.\nRaita:\nRiv agurk og bland den med yoghurt. Hakk mynte og hvitløk og bland det i. Smak til med salt og pepper.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.9, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("indisk", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "En god og rask oppskrift på en kylling tikka masala. Dette er en rett med små smakseksplosjoner som sender tankene til India.",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._exceptions import ElementNotFoundInHtml
from ._utils import normalize_string


class Valdemarsro(AbstractScraper):
    @classmethod
    def host(cls):
        return "valdemarsro.dk"

    def title(self):
        return self.soup.h1.get_text().strip()

    def category(self):
        category_group_element = self.soup.find("div", {"class": "recipe-bar"})

        categories = [
            a.get_text()
            for a in category_group_element.find_all(
                lambda a: "/tag/" not in a.attrs["href"]
            )
        ]
        return ",".join(categories)

    def get_time(self, label):
        metadata_group_element = self.soup.findAll(
            "span", {"class": "recipe-stat-label"}
        )
        time_label_element = next(
            filter(lambda x: label in x.getText(), metadata_group_element), None
        )
        time_text_element = time_label_element.parent.find("strong")
        time_text = time_text_element.getText()

        matched = re.search(
            r"((?P<hours>\d+) timer?\s*)?((?P<minutes>\d+)\s*min.)?", time_text
        )

        minutes = int(matched.groupdict().get("minutes") or 0)
        minutes += 60 * int(matched.groupdict().get("hours") or 0)

        if minutes > 0:
            return minutes
        else:
            raise ElementNotFoundInHtml(label)

    def total_time(self):
        return self.get_time("Tid i alt")

    def cook_time(self):
        return self.get_time("Arbejdstid")

    def yields(self):
        yields_element = self.soup.find("i", {"class": "fa-sort"}).parent
        yields_text = yields_element.getText().strip()
        return yields_text

    def image(self):
        parent_element = self.soup.find("div", {"class": "recipe-image"})
        image_element = parent_element.find(lambda x: not x.has_attr("class"))
        image_url = image_element["src"]
        return image_url

    def ingredients(self):
        ingredient_elements = self.soup.find_all("li", {"itemprop": "recipeIngredient"})

        return [
            normalize_string(paragraph.get_text().strip())
            for paragraph in ingredient_elements
        ]

    def instructions(self):
        instruction_elements = self.soup.find(
            "div", {"itemprop": "recipeInstructions"}
        ).find_all("p")

        instructions_list = [tag.get_text() for tag in instruction_elements]

        return "\n".join(instructions_list)

    def author(self):
        return "Ann-Christine Hellerup Brandt"

    def description(self):
        description_element = self.soup.find("div", {"itemprop": "description"})

        description_paragraph_elements = description_element.find_all(
            lambda tag: tag.name == "p" and " også:" not in tag.get_text(),
            recursive=False,
        )

        description_paragraphs = [
            p.get_text().strip() for p in description_paragraph_elements
        ]

        description = "\n".join(description_paragraphs)

        return description



from .exception_handling import ExceptionHandlingPlugin
from .html_tags_stripper import HTMLTagStripperPlugin
from .normalize_string import NormalizeStringPlugin
from .opengraph_image_fetch import OpenGraphImageFetchPlugin
from .schemaorg_fill import SchemaOrgFillPlugin

__all__ = [
    "ExceptionHandlingPlugin",
    "HTMLTagStripperPlugin",
    "NormalizeStringPlugin",
    "OpenGraphImageFetchPlugin",
    "SchemaOrgFillPlugin",
]



from recipe_scrapers.sweetcsdesigns import SweetCsDesigns
from tests import ScraperTest


class SweetCsDesignsScraper(ScraperTest):

    scraper_class = SweetCsDesigns

    def test_host(self):
        self.assertEqual("sweetcsdesigns.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://sweetcsdesigns.com/the-best-easy-air-fryer-french-fries-recipe/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "The Best Easy Air Fryer French Fries Recipe"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Courtney O'Dell")

    def test_total_time(self):
        self.assertEqual(65, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://sweetcsdesigns.com/wp-content/uploads/2018/12/The-Best-French-Fries-from-an-Air-Fryer-Recipe-Picture-480x480.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 large russet potatoes",
                "2-3 tablespoons olive oil",
                "Sea salt and pepper, to taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Using a mandoline, slice the potatoes into fries. I don't worry too much about the size of fry I am making- some are bigger, and some are smaller. That's fine!\nNext, place your spuds in a nice cool water bath. Completely submerge the fries in water.\nLet fries sit one hour. This helps to remove excess starch and will help the fries crisp up more in the oven.\nPreheat air fryer to 375 degrees.\nAfter an hour, drain the water, and pat fries dry with a paper towel.\nToss with a couple tablespoons of olive oil, salt and pepper.\nAdd fries to bottom of air fryer basket, making sure they are all on the same level (don't stack them on top of each other.)\nCook 13 minutes, until crispy and golden brown.\nPlace on a baking sheet lined with paper towels and a cooling rack over it.\nPlace in warm oven (set to the minimum temperature, not over 250 degrees) and let rest while other batches of fries are cooking.\nServe hot and enjoy.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.kuchniadomowa import KuchniaDomowa
from tests import ScraperTest


class TestKuchniaDomowaScraper(ScraperTest):

    scraper_class = KuchniaDomowa

    def test_host(self):
        self.assertEqual("kuchnia-domowa.pl", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://www.kuchnia-domowa.pl/przepisy/dodatki-do-dan/548-mizeria",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Mizeria", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kuchnia Domowa")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_language(self):
        self.assertEqual("pl-pl", self.harvester_class.language())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://static.kuchnia-domowa.pl/images/content/548/mizeria.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "600 g świeżych ogórków gruntowych (lub długich, szklarniowych)*",
                "300 g gęstej, kwaśnej śmietany 18% lub jogurtu typu greckiego",
                "1 łyżeczka soli",
                "1 łyżka soku z cytryny (lub niepełna łyżka octu jabłkowego)",
                "1 łyżeczka cukru",
                "czarny pieprz mielony",
                "1 łyżka drobno posiekanego koperku",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Ogórki umyć, osuszyć, obrać i pokroić w jak najcieńsze plasterki.\nPlasterki umieścić w misce i posypać 1 łyżeczką soli. Wymieszać i pozostawić na ok. 15 minut.\nW międzyczasie śmietanę przełożyć do miseczki. Przyprawić sokiem z cytryny, cukrem, pieprzem i posiekanym koperkiem. Wymieszać.\nPo 15 minutach odlać wodę, którą puściły ogórki. (Lekko je odcisnąć, ale nie za mocno, aby mizeria nie wyszła za sucha).\nDodać przygotowaną śmietanę i wymieszać.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

# generate generates a new recipe scraper.
import ast
import sys

import requests

from recipe_scrapers._abstract import HEADERS
from recipe_scrapers._utils import get_host_name

template_class_name = "Template"
template_host_name = "example.com"


def generate_scraper(class_name, host_name):
    with open("templates/scraper.py") as source:
        code = source.read()
        program = ast.parse(code)

        state = GenerateScraperState(class_name, host_name, code)
        for node in ast.walk(program):
            if not state.step(node):
                break

        output = f"recipe_scrapers/{class_name.lower()}.py"
        with open(output, "w") as target:
            target.write(state.result())


def generate_scraper_test(class_name, host_name):
    with open("templates/test_scraper.py") as source:
        code = source.read()
        program = ast.parse(code)

        state = GenerateTestScraperState(class_name, host_name, code)
        for node in ast.walk(program):
            if not state.step(node):
                break

        output = f"tests/test_{class_name.lower()}.py"
        with open(output, "w") as target:
            target.write(state.result())


def init_scraper(class_name):
    with open("recipe_scrapers/__init__.py", "r+") as source:
        code = source.read()
        program = ast.parse(code)

        state = InitScraperState(class_name, code)
        for node in ast.walk(program):
            if not state.step(node):
                break

        source.seek(0)
        source.write(state.result())
        source.truncate()


def generate_test_data(class_name, content):
    output = f"tests/test_data/{class_name.lower()}.testhtml"
    with open(output, "wb") as target:
        target.write(content)


class ScraperState:
    def __init__(self, code):
        self.code = code
        self.line_offsets = get_line_offsets(code)
        self.replacer = Replacer(code)

    def result(self):
        return self.replacer.result()

    def _offset(self, node):
        return self.line_offsets[node.lineno - 1] + node.col_offset

    def _replace(self, replacement_text, start, length):
        self.replacer.replace(replacement_text, start, length)


class GenerateScraperState(ScraperState):
    def __init__(self, class_name, host_name, code):
        super().__init__(code)
        self.class_name = class_name
        self.host_name = host_name

    def step(self, node):
        if isinstance(node, ast.ClassDef) and node.name == template_class_name:
            offset = self._offset(node)
            segment_end = self.code.index(template_class_name, offset)
            self._replace(self.class_name, segment_end, len(template_class_name))

        if isinstance(node, ast.Constant) and node.value == template_host_name:
            offset = self._offset(node)
            segment_end = self.code.index(template_host_name, offset)
            self._replace(self.host_name, segment_end, len(template_host_name))

        return True


class GenerateTestScraperState(ScraperState):
    def __init__(self, class_name, host_name, code):
        super().__init__(code)
        self.class_name = class_name
        self.host_name = host_name
        self.module_name = class_name.lower()
        self.template_module_name = template_class_name.lower()

    def step(self, node):
        if (
            isinstance(node, ast.ImportFrom)
            and node.module == f"recipe_scrapers.{self.template_module_name}"
        ):
            offset = self._offset(node)
            module_name_segment_end = self.code.index(self.template_module_name, offset)
            self._replace(
                self.module_name,
                module_name_segment_end,
                len(self.template_module_name),
            )
            class_name_segment_end = self.code.index(template_class_name, offset)
            self._replace(
                self.class_name, class_name_segment_end, len(template_class_name)
            )

        if (
            isinstance(node, ast.ClassDef)
            and node.name == f"Test{template_class_name}Scraper"
        ):
            offset = self._offset(node)
            segment_end = self.code.index(template_class_name, offset)
            self._replace(self.class_name, segment_end, len(template_class_name))

        if (
            isinstance(node, ast.Assign)
            and isinstance(node.value, ast.Name)
            and node.value.id == template_class_name
        ):
            offset = self._offset(node)
            segment_end = self.code.index(template_class_name, offset)
            self._replace(self.class_name, segment_end, len(template_class_name))

        if isinstance(node, ast.Constant) and node.value == template_host_name:
            offset = self._offset(node)
            segment_end = self.code.index(template_host_name, offset)
            self._replace(self.host_name, segment_end, len(template_host_name))

        return True


class InitScraperState(ScraperState):
    def __init__(self, class_name, code):
        super().__init__(code)
        self.class_name = class_name
        self.module_name = class_name.lower()
        self.state = "import"
        self.last_node = None

    def step(self, node):
        if self.state == "import":
            return self._import(node)
        elif self.state == "init":
            return self._init(node)
        else:
            return False

    def _import(self, node):
        if isinstance(node, ast.Module) or isinstance(node, ast.Import):
            return True

        if isinstance(node, ast.ImportFrom):
            if node.module > self.module_name and node.level > 0:
                offset = self._offset(node)
                import_statement = (
                    f"\nfrom .{self.module_name} import {self.class_name}"
                )
                self._replace(import_statement, offset, 0)
                self.state = "init"
            self.last_node = node
        elif isinstance(self.last_node, ast.ImportFrom):
            offset = (
                self.line_offsets[self.last_node.lineno - 1]
                + self.last_node.end_col_offset
            )
            segment_end = self.code.index("\n", offset)
            import_statement = f"\nfrom .{self.module_name} import {self.class_name}"
            self._replace(import_statement, segment_end, 0)
            self.state = "init"
            return self._init(node)

        return True

    def _init(self, node):
        if isinstance(node, ast.Assign):
            for target in node.targets:
                if (
                    hasattr(target, "id")
                    and target.id == "SCRAPERS"
                    and isinstance(node.value, ast.Dict)
                ):
                    for key in node.value.keys:
                        if (
                            isinstance(key, ast.Call)
                            and isinstance(key.func, ast.Attribute)
                            and isinstance(key.func.value, ast.Name)
                        ):
                            if key.func.value.id > self.class_name:
                                offset = self._offset(key)
                                init_statement = f" {self.class_name}.host(): {self.class_name},\n   "
                                self._replace(init_statement, offset, 0)
                                return False
                            self.last_node = key

            if isinstance(self.last_node, ast.Call):
                offset = (
                    self.line_offsets[self.last_node.lineno - 1]
                    + self.last_node.end_col_offset
                )
                segment_end = self.code.index("\n", offset)
                init_statement = f"\n    {self.class_name}.host(): {self.class_name},"
                self._replace(init_statement, segment_end, 0)
                return False

        return True


class Replacer:
    def __init__(self, code):
        self.code = code
        self.delta = 0
        self.replacements = []

    def replace(self, replacement_text, start, length):
        self.replacements.append((replacement_text, start, length))

    def result(self):
        code = self.code
        for (replacement_text, start, length) in self.replacements:
            start = start + self.delta
            end = start + length
            code = code[:start] + replacement_text + code[end:]
            self.delta += len(replacement_text) - length

        return code


def get_line_offsets(code):
    offset = 0
    indices = [0]
    try:
        while True:
            index = code.index("\n", offset)
            indices.append(index)
            offset = index + 1
    except ValueError:
        return indices


def main():
    if len(sys.argv) < 3:
        print("Usage: generate.py ScraperClassName url")
        exit(1)

    class_name = sys.argv[1]
    url = sys.argv[2]
    host_name = get_host_name(url)
    testhtml = requests.get(url, headers=HEADERS).content

    generate_scraper(class_name, host_name)
    generate_scraper_test(class_name, host_name)
    generate_test_data(class_name, testhtml)
    init_scraper(class_name)


if __name__ == "__main__":
    main()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class KennyMcGovern(AbstractScraper):
    @classmethod
    def host(cls):
        return "kennymcgovern.com"

    def title(self):
        return self.soup.find("div", {"class": "wprm-recipe-name"}).get_text()

    def total_time(self):
        return get_minutes(
            self.soup.find("span", {"class": "wprm-recipe-total_time"}).parent
        )

    def yields(self):
        yields = self.soup.find("span", {"class": "wprm-recipe-servings"}).get_text()

        return get_yields("{} servings".format(yields))

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"class": "wprm-recipe-ingredient"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.findAll(
            "div", {"class": "wprm-recipe-instruction-text"}
        )

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



# mypy: disallow_untyped_defs=False
import requests

from ._abstract import HEADERS, AbstractScraper
from ._schemaorg import SchemaOrg
from ._utils import url_path_to_dict


class Woolworths(AbstractScraper):
    def __init__(self, url, proxies=None, timeout=None, *args, **kwargs):
        super().__init__(url=url, *args, **kwargs)

        target = url_path_to_dict(url)["path"].split("/")[-1]
        data_url = f"https://foodhub.woolworths.com.au/content/woolworths-foodhub/en/{target}.model.json"

        self.page_data = (
            requests.get(
                data_url,
                headers=HEADERS,
                proxies=proxies,
                timeout=timeout,
            )
            .json()
            .get(":items")
            .get("root")
            .get(":items")
            .get("recipe_seo_data")
        )
        self.schema = SchemaOrg(self.page_data, raw=True)

    @classmethod
    def host(cls):
        return "woolworths.com.au"

    def canonical_url(self):
        return self.url

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def cook_time(self):
        return self.schema.cook_time()

    def prep_time(self):
        return self.schema.prep_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def nutrients(self):
        return self.schema.nutrients()

    def language(self):
        return "en-AU"

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def author(self):
        return self.schema.author()

    def cuisine(self):
        return self.schema.cuisine()

    def site_name(self):
        return "Woolworths | Fresh Ideas For You"



from recipe_scrapers.cookieandkate import CookieAndKate
from tests import ScraperTest


class TestCookieAndKateScraper(ScraperTest):

    scraper_class = CookieAndKate

    def test_host(self):
        self.assertEqual("cookieandkate.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://cookieandkate.com/broccoli-cheddar-spinach-frittata/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Broccoli, Cheddar & Spinach Frittata"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Cookie and Kate")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "8 SimplyNature Organic Cage Free Eggs",
                "½ cup milk of choice",
                "2 small-to-medium cloves garlic, pressed or minced",
                "½ teaspoon sea salt, divided",
                "Freshly ground black pepper",
                "1 cup freshly grated cheddar cheese, divided",
                "1 tablespoon SimplyNature Organic Extra Virgin Olive Oil, more as needed",
                "1 small yellow onion, chopped",
                "⅓ cup water",
                "2 cups thinly sliced broccoli florets",
                "2 cups SimplyNature Organic Baby Spinach, roughly chopped",
                "⅓ cup thinly sliced green onions",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 425 °F. In a large bowl, whisk together the eggs, milk, garlic, ¼ teaspoon of the salt and about 5 twists of freshly ground black pepper until well blended. Then whisk in about half of the cheese, reserving the other half for later.\nIn a 10-inch, well-seasoned cast iron skillet or oven-safe sauté pan, warm the olive oil over medium heat until shimmering. Add the onion and the remaining ¼ teaspoon salt. Cook, stirring frequently, until the onion is tender and translucent, about 3 to 5 minutes.\nAdd the broccoli and water to the pan, then cover it with a lid (or a baking sheet) and steam the mixture until the broccoli is brighter green and easily pierced by a fork, about 2 to 3 minutes. Uncover, and add the spinach and green onions. Cook, stirring constantly, until the spinach has wilted, about 30 to 60 seconds.\nArrange the mixture in an even layer across the skillet. Whisk the egg mixture one last time and pour it into the pan. Sprinkle the frittata with the remaining cheese. Put the pan in the oven and bake until you can shimmy the pan by the handle (careful, it’s hot!) and see that the middle is just barely set, about 12 to 15 minutes.\nOnce the frittata is done baking, let it rest for 5 to 10 minutes before slicing it into 6 large or 8 smaller wedges. Serve immediately. Leftover frittata will keep well, covered and refrigerated, for up to 3 days. Enjoy chilled or gently reheat.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.9, self.harvester_class.ratings())



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class OmnivoresCookbook(AbstractScraper):
    @classmethod
    def host(cls):
        return "omnivorescookbook.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = self.soup.find(
            "ul", {"class": "wprm-recipe-ingredients"}
        ).find_all("li")
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.soup.head.find("meta", {"property": "og:description"})["content"]



from recipe_scrapers.ig import IG
from tests import ScraperTest


class TestIGScraper(ScraperTest):

    scraper_class = IG

    def test_host(self):
        self.assertEqual("receitas.ig.com.br", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://receitas.ig.com.br/estrogonofe-de-cogumelos/4e7b634e7bb4e2ad5c000054.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Estrogonofe de cogumelos")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Fabiana Badra")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://i0.statig.com.br/bancodeimagens/4y/4c/rs/4y4crseb7npraepfzuhc8iimy.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 colheres (sopa) de azeite",
                "1 cebola roxa cortada em cubos",
                "2 dentes de alho espremidos",
                "500 g de cogumelos variados (shiitake, shimeji, portobelo e champignons)",
                "2 colheres (sopa) de suco de limão",
                "1 colher (sopa) de catchup",
                "400 ml de caldo de legumes",
                "1 colher (sopa) de amido de milho dissolvido em água fria",
                "100 g de creme de leite",
                "½ xícara (chá) de iogurte desnatado",
                "3 colheres (sopa) de salsinha picada",
                "Arroz branco para acompanhar",
                "Batata palha",
                "",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Em uma panela com o azeite, refogue a cebola e o alho até que a cebola comece a murchar. Junte os cogumelos, o limão, e o catchup. Cozinhe por 5 a 10 minutos. Adicione o caldo e cozinhe por mais 5 minutos. Junte a maisena e deixe ferver até engrossar ligeiramente. Por último, adicione o creme de leite, tempere e junte a salsinha. Sirva com arroz e batata palha.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.emmikochteinfach import EmmiKochtEinfach
from tests import ScraperTest


class TestEmmiKochtEinfachScraper(ScraperTest):
    scraper_class = EmmiKochtEinfach

    def test_host(self):
        self.assertEqual("emmikochteinfach.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Emmi", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Klassisches Rindergulasch ganz einfach", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://emmikochteinfach.de/wp-content/uploads/2022/02/seo_Gulasch-Nah-200.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "600 g Rindergulasch (ca. 3x3 cm große Stücke, gegebenenfalls selbst kleiner schneiden)",
                "400 ml Rinderfond (selbstgemacht oder aus dem Glas)",
                "300 g Schalotten (geschält und halbiert)",
                "200 ml trockener Rotwein, einen den Du gerne trinkst (alternativ roter 100% Traubendirektsaft )",
                "40 g Tomatenmark",
                "3 EL Butterschmalz (z.B. Buttaris; Alternativ Pflanzenöl )",
                "1 Knoblauchzehe (geschält, klein geschnitten)",
                "1-2 TL Zitronenabrieb (unbehandelt bzw. Bio) (kannst Du auch weggelassen )",
                "2 TL Paprikapulver, edelsüß",
                "1/2 TL Majoran, getrocknet",
                "1 Msp. Cayennepfeffer",
                "1 Prise Salz",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """
Das Gulasch-Fleisch bitte ca. eine halbe Stunde vor der Zubereitung aus dem Kühlschrank nehmen, damit es Zimmertemperatur annehmen kann und nicht eiskalt gebraten wird. Wenn Du das Fleisch selbst schneidest, erst Scheiben quer zur Fleischfaser und daraus wiederum Würfel schneiden. Eine ideale Größe ist ca. 3x3 cm (Walnussgröße). Die 600 g Fleisch nur mäßig salzen, wenn überhaupt nur mit 1 Prise Salz kurz vor dem Anbraten.
Jetzt schälst Du die 300 g Schalotten und halbierst sie, besonders große Exemplare kannst Du auch vierteln. 1 EL Butterschmalz in einem Bräter / Schmortopf erhitzen und darin die Schalotten goldgelb anbraten. Herausnehmen und zur Seite stellen.
Im Anschluss wieder 1 EL Butterschmalz im Bräter erhitzen und bei sehr hoher Temperatur das Fleisch portionsweise anbraten. Erst nach ca. 1 Minute das erste Mal wenden, damit nicht zu viel Hitze vom Boden entweicht, das Fleisch eine schöne Bräune annimmt und sich Röstaromen bilden. Die Fleischportionen entsprechend zur Seite stellen. HINWEIS: Meine Empfehlung ist das Fleisch auf jeden Fall portionsweise anzubraten, auch wenn es etwas Geduld erfordert. Wenn das gesamte Gulaschfleisch auf einmal in den Topf kommt, kann der Boden in der Regel die Hitze nicht halten und das Fleisch köchelt mehr als zu braten.
Nun kommt das gesamte Fleisch zurück in den Bräter zur letzten Fleischportion zurück, ebenfalls die Schalotten sowie 40 g Tomatenmark. Das Tomatenmark gut unterrühren damit es kurz mit rösten kann.
Mit 200 ml trockenem Rotwein ablöschen und den Rotwein zu 2/3 verkochen lassen, sprich auf ungefähr 1/3 reduzieren.
In der Zwischenzeit 1 Knoblauchzehe schälen und in feine Würfel schneiden. Die Zitrone heiß abwaschen, abtrocknen und die Schale mit einer feinen Reibe abreiben für ca. 1 bis 2 TL Zitronenabrieb. Wenn der Rotwein reduziert ist, den Knoblauch, 1-2 TL Zitronenabrieb, 2 TL Paprikapulver, 1/2 TL Majoran und 1 Messerspitze Cayennepfeffer hinzufügen und kurz unterrühren.TIPP: Das Gulasch erhält durch den Zitronenabrieb ein feines Zitronenaroma. Wenn Du das nicht möchtest, kannst Du den Abrieb auch einfach weglassen.
Das Fleisch und die Schalotten nun mit 400 ml Rinderfond ablöschen und mit Deckel für ca. 1,5 Stunden bei geringer Hitze auf der Herdplatte garen. Nach einer Stunde mal nachsehen, ob Dir die Soße zu flüssig erscheint. Falls ja, kannst Du das Gulasch die letzten 30 Minuten ohne Deckel weiter garen. Am Ende der Garzeit das Gulasch nach Belieben abschmecken und gegebenenfalls eindicken.
Ich wünsche Dir einen guten Appetit! Lass Dir mein klassisches Rindergulasch Rezept gut schmecken.
        """.strip(),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.99, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "Mein Familienrezept ist ein echter Klassiker. Gulasch kochen geht einfacher als man denkt. Mit zartem Fleisch, leckeren Röstaromen und einer sämigen Soße wird es auch Dir gelingen.",
            self.harvester_class.description(),
        )

    def test_total_time(self):
        return self.assertEqual(120.0, self.harvester_class.total_time())

    def test_cook_time(self):
        return self.assertEqual(90.0, self.harvester_class.cook_time())

    def test_prep_time(self):
        return self.assertEqual(30.0, self.harvester_class.prep_time())



from recipe_scrapers.madensverden import MadensVerden
from tests import ScraperTest


class TestMadensVerdenScraper(ScraperTest):

    scraper_class = MadensVerden

    def test_host(self):
        self.assertEqual("madensverden.dk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "Rabarbertrifli",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Dessert",
            self.harvester_class.category(),
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(15, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(15, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://madensverden.dk/wp-content/uploads/2017/08/billederesultat-for-rabarbertrifli.jpg",
            self.harvester_class.image(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {"calories": "270 kcal", "servingSize": "1 person"},
            self.harvester_class.nutrients(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "400 gram rabarber",
                "75 gram sukker",
                "10 gram vaniljesukker",
                "2 pasteuriserede æggeblommer (1 bæger)",
                "40 gram sukker",
                "10 gram majsstivelse",
                "2,5 deciliter sødmælk",
                "2,5 deciliter piskefløde",
                "100 gram makroner",
                "40 gram mandelflager",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Start med at lave rabarberkompot. Rabarber snittes i tynde skiver, som koges i en kasserolle sammen med sukker og vaniljesukker. Kogetiden er cirka 10 minutter, og du skal ikke tilsætte vand, da rabarberne kaster rigeligt med væske af sig. Lad kompotten køle af.\nCremen laves ved at piske æggeblommer sammen med sukker og majsstivelse i en kasserolle. Koges op med sødmælken indtil den har den rette konsistens, og du skal piske i den undervejs så cremen ikke brænder på. Lad den køle lidt af.\nFløden piskes til en let flødeskum.\nAnret nu trifli med rabarber i portionsglas.\nFørst med et lag creme nederst, så knuste makroner og ovenpå det den lækre rabarberkompot.\nSlut af med flødeskum og pynt med mandelflager.\nStil rabarbertriflierne i køleskabet, og lad dem trække i mindst en time før servering.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.77, self.harvester_class.ratings())

    def test_author(self):
        self.assertEqual("Holger Rørby Madsen", self.harvester_class.author())

    def test_cuisine(self):
        self.assertEqual("Dansk", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Rabarbertrifli er en populær dessert, der laves med knuste makroner og flødeskum. Den lækre trifli med rabarber kan laves god tid i forvejen.",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("da-DK", self.harvester_class.language())

    def test_site_name(self):
        self.assertEqual("Madens Verden", self.harvester_class.site_name())



from recipe_scrapers.ditchthecarbs import DitchTheCarbs
from tests import ScraperTest


class TestDitchTheCarbs(ScraperTest):

    scraper_class = DitchTheCarbs

    def test_host(self):
        self.assertEqual("ditchthecarbs.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.ditchthecarbs.com/easy-keto-hamburger-buns-almond-flour/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Keto Hamburger Buns Recipe")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Libby Jenkinson")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.ditchthecarbs.com/wp-content/uploads/2021/04/Easy-Keto-Hamburger-Buns-almond-flour-1200x1200-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 egg",
                "1 tsp apple cider vinegar",
                "¼ cup ground flaxseed/linseed (ground)",
                "¾ cup almond meal/flour",
                "1¾ cup pre-shredded/grated mozzarella",
                "2 tbsp cream cheese",
                "1 tsp baking powder",
                "1 tsp extra virgin olive oil",
                "1 tsp sesame seeds",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat your oven to 375°F/190°C. In a small bowl mix together your egg and apple cider vinegar. Set aside until ready to use.\nMix\nAdd your flax seen to a grinder or blender, coarsely grind. Pour the flax seed in a microwave safe bowl. Add the almond flour and shredded mozzarella cheese to the bowl and mix. Then add your cream cheese to the bowl. There is no need to mix the cream cheese in just yet.\nMicrowave\nPlace your bowl with the cheese in the microwave. Melt your cheese for 1 minute, remove the bowl and mix with a silicone spatula. If your cheese is not completely melted yet microwave for 30 more seconds until your cheese is completely melted.\nMix\nWhile your mozzarella dough is still hot add your baking powder. Mix and fold together. Add the egg/vinegar mixture to your dough. Fold in with your silicone spatula.\nRoll your dough into a ball and cut into 4 equal sections. Roll each section into a ball and then flatten the top and bottom of the ball to shape into a hamburger bun.\nPlace on a baking sheet lined with baking paper. Brush the olive oil over your buns and sprinkle the tops with sesame seeds. Bake in the oven at 375°F/190°C for 12-15 minutes. Your burger buns will be done when the tops are golden brown and the dough is cooked in the centre.\nBake\nRemove the buns from the oven and let them cool on a wire rack or towel for 5 minutes before slicing and serving.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields


class PingoDoce(AbstractScraper):
    @classmethod
    def host(cls):
        return "pingodoce.pt"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return get_minutes(self.soup.findAll("div", {"class": "info"})[0])

    def yields(self):
        return get_yields(self.soup.findAll("div", {"class": "info"})[2])

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        instructions = self.soup.findAll("div", {"class": "step-description"})
        return "\n".join([i.get_text() for i in instructions])

    def ratings(self):
        return self.schema.ratings()

    def description(self):
        return self.schema.description()



from recipe_scrapers.livelytable import LivelyTable
from tests import ScraperTest


class TestLivelyTableScraper(ScraperTest):

    scraper_class = LivelyTable

    def test_host(self):
        self.assertEqual("livelytable.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://livelytable.com/easy-chipotle-shrimp-tacos/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Easy Chipotle Shrimp Tacos")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kaleigh")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://livelytable.com/wp-content/uploads/2017/05/chipotle-shrimp-tacos-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 lb raw large shrimp, peeled and deveined",
                "1/2 tsp chipotle powder",
                "1/2 tsp garlic powder",
                "1/2 tsp ground cumin",
                "1 tbsp olive or avocado oil",
                "Juice from 1/2 lime",
                "1/2 jalapeño, seeded and diced",
                "1/4 cup cilantro leaves",
                "2/3 cup nonfat plain greek yogurt",
                "Juice from 1/2 lime",
                "6 corn tortillas, warmed",
                "Shredded purple cabbage",
                "Cilantro",
                "Crumbled cotija cheese",
                "Lime wedges",
                "Guacamole or diced avocado",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Place shrimp in a medium, nonreactive bowl.\nIn a small bowl, combine chipotle, garlic powder, and cumin. Sprinkle over shrimp. Add oil and lime juice and stir well until shrimp are evenly coated. Set aside while you prepare the cilantro jalapeño sauce.\nIn a food processor or blender, combine jalapeño, cilantro, yogurt, and lime juice. Process until smooth and pour into a small bowl for serving.\nPrepare remaining taco toppings.\nHeat a large non-stick skillet over medium heat. Add shrimp and cook about 3 minutes per side or until cooked through.\nServe with warm corn tortillas and top as desired. Enjoy!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.heinzbrasil import HeinzBrasil
from tests import ScraperTest


class TestHeinzBrasilScraper(ScraperTest):

    scraper_class = HeinzBrasil

    def test_host(self):
        self.assertEqual("heinzbrasil.com.br", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.heinzbrasil.com.br/recipe/100149100002/chili-com-carne",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chili com carne")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kraft Heinz")

    def test_image(self):
        self.assertEqual(
            "//d36rz30b5p7lsd.cloudfront.net/heinzbrasilbr/recipes/img/1e1fccb2d38d8b3e3611aa5c99706523.jpeg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 kg de carne moída",
                "1 cebola picada",
                "1 pimentão vermelho cortado em cubos",
                "1 pimentão verde cortado em cubos",
                "100 g de manteiga",
                "50 ml azeite",
                "20 g de especiarias mexicanas (pimentão, cominho, coentro...)",
                "100 ml de café forte",
                "150 g de milho crocante, escorrido",
                "200 g de feijão lavado e escorrido (feijão vermelho da América do Sul)",
                "2 tomates grandes cortados em cubos",
                "400 ml de Ketchup Heinz",
                "200 g de queijo gruyère ralado fininho",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Frite a carne, a cebola e os pimentões na manteiga com o azeite, mexendo bem. Despeje as especiarias mexicanas e o café. Adicione o milho, o feijão, os tomates picados e o Ketchup Heinz. Cozinhe por 5 minutos. Sirva com um bom pedaço de pão e gruyère.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.simpleveganista import SimpleVeganista
from tests import ScraperTest


class TestSimpleVeganistaScraper(ScraperTest):

    scraper_class = SimpleVeganista

    def test_host(self):
        self.assertEqual("simple-veganista.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Julie | The Simple Veganista", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Vegan Jambalaya", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Entree", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://simple-veganista.com/wp-content/uploads/2022/02/easy-vegan-jambalaya-recipe-5-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tablespoon olive oil or 1/4 cup water",
                "1 medium onion, diced",
                "4 cloves garlic, minced",
                "3 large bell peppers (green, red & yellow), seeded and diced",
                "2 large celery ribs, diced",
                "2 teaspoons smoked paprika",
                "1 teaspoon EACH dried thyme + oregano",
                "1/4 – 1/2 teaspoon cayenne or red pepper flakes",
                "1 can (14oz) crushed tomatoes",
                "1 1/2 cups dry long-grain rice (for grain-free see notes)",
                "1 – 2 bay leaves",
                "3 1/2 – 4 cups low-sodium vegetable broth",
                "1 can (14oz) red kidney beans, drained and rinsed",
                "3 – 4 vegan sausages (about 14 oz), sliced and cooked (or extra can of beans)",
                "1 teaspoon pink salt, or to taste",
                "fresh cracked pepper, to taste",
                "sliced scallions (green onions)",
                "chopped parsley",
                "dash of hot sauce (Tabasco, Frank’s, or your favorite)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Vegan sausage\nIf using plant-based sausage, you’ll want to cook it first. Slice it into bite-size pieces and cook in a skillet with 1 tablespoon oil, over medium heat until browned, about 3 – 5 minutes on each side. Transfer to a small plate lined with a paper napkin to soak up excess oil.\nSaute\nIn a large pan, heat oil/water over medium-high heat, add the onion, garlic, celery, and bell peppers, saute for 5 minutes. Add the smoked paprika, thyme, oregano, and cayenne, and saute for 1 minute more.\nSimmer\nAdd the crushed tomatoes, rice, bay leaves, and broth, bring to a boil, reduce heat, cover askew, and simmer on low for 20 – 25 minutes, stirring every 7 minutes or so to keep the rice from sticking to the bottom of the pan. In the last 5 minutes, add the beans and/or sausage and continue to cook until warmed through. Remove bay leaves, season with salt and pepper.\nServe\nSpoon into serving bowls and top with garnish of choice.\nServes 6 – 8\nStore\nLeftovers can be stored for up to 5 days in the refrigerator. For longer storage, keep in the freezer for up to 2 months. Let thaw before reheating on the stovetop or in the microwave.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Creole", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Made in 1 pot, this easy Louisiana Vegan Jambalaya is loaded with healthy veggies and plant-based protein for a delicious Creole-style rice dish that even the picky eaters will love!",
            self.harvester_class.description(),
        )



from recipe_scrapers.nourishedbynutrition import NourishedByNutrition
from tests import ScraperTest


class TestNourishedByNutritionScraper(ScraperTest):

    scraper_class = NourishedByNutrition

    def test_host(self):
        self.assertEqual("nourishedbynutrition.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://nourishedbynutrition.com/healthy-salsa-verde-chicken-enchiladas-dairy-free/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Salsa Verde Chicken Enchiladas")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Jessica Bippen")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://nourishedbynutrition.com/wp-content/uploads/2020/08/Salsa-Verde-Chicken-Enchiladas-8.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups shredded cooked chicken breast",
                "2 cups salsa verde, divided",
                "5 ounces spinach",
                "½ teaspoon garlic powder",
                "½ teaspoon ground cumin",
                "salt and pepper to taste",
                "8 tortillas (cassava flour or corn, flour, wheat, etc)",
                "½ cup Siete queso blanco (see notes for substitutions)",
                "½ avocado, thinly sliced",
                "¼ cilantro leaves",
                "2 tablespoons finely diced red onion",
                "1 lime, cut into wedges",
                "2-3 thinly sliced radishes",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 375F degrees.\nHeat a medium skillet over medium heat. Add the spinach and cover with a lid. Let spinach wilt. This should only take about 2-3 minutes. Remove from the heat and set aside.\nIn a bowl, combine the shredded chicken with the garlic powder, cumin, ½ cup of the salsa verde, and wilted spinach. Toss to coat.\nWrap them in a damp paper towel and microwave for 20-40 seconds to loosen them up so that they are easier to roll. You can also heat them directly the gas burner or in a saute pan one at a time. Pour ½ cup of the salsa verde on the bottom of a 9×13 baking dish, spread to coat the bottom.\nTo assemble the enchiladas, place about ¼ cup of chicken mixture on a tortilla. Gently roll the tortilla and place it seam side down into the prepared baking dish. Repeat with the remaining tortillas and chicken.\nPour the remaining 1 cup of salsa over the top of the enchiladas so they’re completely covered. Add the vegan queso blanco or shredded cheese of choice on top, if desired. Cover with foil. Bake for 25 minutes. Remove from oven and let sit for 5-10 minutes before topping with garnishes. Enjoy warm.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.mindmegette import Mindmegette
from tests import ScraperTest


class TestMindmegetteScraper(ScraperTest):

    scraper_class = Mindmegette

    def test_host(self):
        self.assertEqual("www.mindmegette.hu", self.harvester_class.host())

    def test_language(self):
        self.assertEqual("hu", self.harvester_class.language())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.mindmegette.hu/tepsis-krumpli-ceklaval-es-repaval.recept/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Tepsis krumpli céklával és répával"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "MME")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertListEqual(
            [
                "6 db burgonya",
                "2 db cékla",
                "8 db répa",
                "4 db lilahagyma",
                "6 gerezd fokhagyma",
                "friss rozmaring",
                "só",
                "bors",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "A zöldségeket megpucoljuk és megmossuk.\n"
            + "A burgonyát vastagabb szeletekre vágjuk, a répákat hosszában elfelezzük, a lilahagymát félbe vagy negyedbe vágjuk, a céklát szeleteljük, a fokhagymát egészben hagyjuk.\n"
            + "Az összes, előkészített zöldséget egy tepsibe tesszük, meglocsoljuk olívaolajjal, sózzuk és borsozzuk, kézzel összeforgatjuk az egészet, majd friss rozmaringot teszünk rá.\n"
            + "Lefedjük alufóliával, 180 fokon 20 percig sütjük, majd fólia nélkül, amíg minden zöldség meg nem puhul. Grill funkció esetén az utolsó 5 percben meg is piríthatjuk. Sült húsok mellé kiváló, laktató köret.",
            self.harvester_class.instructions(),
        )

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.mindmegette.hu/images/283/O/tepsis-ceklas.jpg",
            self.harvester_class.image(),
        )



from recipe_scrapers.food52 import Food52
from tests import ScraperTest


class TestFood52(ScraperTest):

    scraper_class = Food52

    def test_host(self):
        self.assertEqual("food52.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://food52.com/recipes/84961-pomegranate-chicken-wings-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Sticky Pomegranate & Black Pepper Chicken Wings",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Sohla El-Waylly")

    def test_total_time(self):
        self.assertEqual(1520, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 to 2 1/4 pounds chicken wings",
                "1 tablespoon kosher salt",
                "1 tablespoon granulated sugar",
                "2 teaspoons baking powder",
                "1/2 teaspoon MSG (optional)",
                "2 tablespoons neutral oil, such as grapeseed or canola",
                "1/4 cup granulated sugar",
                "1/2 cup pomegranate molasses",
                "2 tablespoons freshly ground black pepper",
                "2 teaspoons kosher salt",
                "2 teaspoons ground cinnamon",
                "1 teaspoon freshly ground nutmeg",
                "1/2 teaspoon toasted walnut halves, roughly chopped",
                "Flaky salt (optional)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Line a sheet pan with foil (for easy cleanup) and set an ovenproof wire rack inside.",
                    "Brine the wings: Pat the wings dry with a paper towel and place in a large bowl. In a small bowl, stir together the kosher salt, sugar, baking powder, and MSG (if using). Sprinkle this mixture over the wings and toss until evenly coated. Arrange the wings on the wire rack and refrigerate uncovered at least 6 hours or preferably overnight, flipping the wings halfway through. (Wash the bowl and keep it handy—we’ll be using it again.)",
                    "When you’re ready to roast, heat the oven to 350°F.",
                    "The first roast: Transfer the wings to the big bowl, toss with the oil until evenly coated, then return them to the wire rack. Roast the wings, flipping them once halfway through, until the skin is lightly browned, the flesh of the drumettes is starting to pull away from the bones, and the fat from the skin has mostly rendered, about 1 hour.",
                    "Meanwhile, prepare the glaze: Wash the large bowl. Add the sugar and pour 2 tablespoons of boiling water on top, whisking until the sugar dissolves. Add the pomegranate molasses, black pepper, salt, cinnamon, and nutmeg, whisking until combined.",
                    "The second roast: Remove the wings from the oven and increase the temperature to 375°F. Transfer the wings to the bowl with the glaze and toss to coat. Use tongs to return the wings to the rack, reserving any remaining glaze in the bowl, and roast until lightly charred and glossy, about 20 minutes.",
                    "Once the wings are out of the oven, transfer them to the bowl and toss again in the remaining glaze. Transfer the wings to a platter and garnish with the walnuts. Sprinkle with flaky salt if desired.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



import importlib
import os
from typing import Any


class RecipeScraperSettings:
    """
    Allowing users to configure package settings when using recipe-scraper
    without the need of touching the core package.

    Similar to django's settings module but simpler.

    Users can write settings in simple python format and using python expressions.
    User defined settings can be saved in whatever python settings module / .py file
    and then used by changing the env variable
    "RECIPE_SCRAPERS_SETTINGS" to point to them:
    os.environ["RECIPE_SCRAPERS_SETTINGS"] = "path.to.my.custom.settings.file" [py]

    will make the package start using the user-defined settings instantly.

    Access package's settings with

    from recipe_scrapers.settings import settings
    # settings.LOG_LEVEL
    # settings.PLUGINS etc. (check recipe_scrapers/settings/default.py for more info)

    Users can easily add Plugins of their own and add/test extra scraper functionality
    as they find fit.
    """

    def __init__(self, *args: Any, **kwargs: Any):
        self._configured = False
        self._user_settings = False
        super().__init__(*args, **kwargs)

    def __getattribute__(self, item: Any) -> Any:
        if item.startswith("_"):
            return super().__getattribute__(item)

        # if settings haven't yet been configured or RECIPE_SCRAPERS_SETTINGS
        # environment has been changed, reconfigure settings before returning
        user_defined_settings = os.environ.get("RECIPE_SCRAPERS_SETTINGS", False)
        if not self._configured or user_defined_settings != getattr(
            self, "_user_settings"
        ):
            self._configure()
            return super().__getattribute__(item)

        return super().__getattribute__(item)

    def _configure(self) -> None:
        # configure the default settings by default
        if not getattr(self, "_configured"):
            default_settings = importlib.import_module(
                "recipe_scrapers.settings.default"
            )
            for item in dir(default_settings):
                if item.isupper():
                    setattr(self, item, getattr(default_settings, item))
            self._configured = True

        # if user-specific given given, overwrite the default ones
        user_settings = os.environ.get("RECIPE_SCRAPERS_SETTINGS", False)
        if user_settings != getattr(self, "_user_settings"):
            setattr(self, "_user_settings", user_settings)
            user_settings = importlib.import_module(user_settings)  # type: ignore [arg-type,assignment]
            for item in dir(user_settings):
                if item.isupper():
                    setattr(self, item, getattr(user_settings, item))


settings = RecipeScraperSettings()


__all__ = ["settings"]



# mypy: disallow_untyped_defs=False
class RecipeScrapersExceptions(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(message)

    def __str__(self):
        return f"recipe-scrapers exception: {self.message}"


class WebsiteNotImplementedError(RecipeScrapersExceptions):
    """Error when website is not supported by this library."""

    def __init__(self, domain):
        self.domain = domain
        message = f"Website ({self.domain}) not supported."
        super().__init__(message)


class NoSchemaFoundInWildMode(RecipeScrapersExceptions):
    """Error when wild_mode fails to locate schema at the url"""

    def __init__(self, url):
        self.url = url
        message = f"No Recipe Schema found at {self.url}."
        super().__init__(message)


class ElementNotFoundInHtml(RecipeScrapersExceptions):
    """Error when we cannot locate the HTML element on the page"""

    def __init__(self, element):
        self.element = element
        message = (
            "Element not found in html (self.soup.find returned None). Check traceback."
        )
        super().__init__(message)


class SchemaOrgException(RecipeScrapersExceptions):
    """Error in parsing or missing portion of the Schema.org data org the page"""

    def __init__(self, message):
        super().__init__(message)



from recipe_scrapers.saveur import Saveur
from tests import ScraperTest


class TestSaveurScraper(ScraperTest):

    scraper_class = Saveur

    def test_host(self):
        self.assertEqual("saveur.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(None, self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Baked Saffron Yogurt Rice with Chicken (Tahcheen-e Morgh)",
            self.harvester_class.title(),
        )

    def test_total_time(self):
        self.assertEqual(285, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.saveur.com/uploads/2019/03/07/httpsrecipes.saveur.comuploadsimagesabb46018d2ab45231975b33416325e96ca3afeab.jpeg?auto=webp",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 cups extra-long white basmati rice (1 lb. 4 oz.)",
                "1⁄4 cup plus 2 Tbsp. kosher salt, divided, plus more as needed",
                "1⁄4 tsp. saffron, lightly ground between your fingers, plus more for sprinkling",
                "3 tbsp. extra-virgin olive oil",
                "1 large yellow onion, halved and sliced ¼-inch-thick",
                "4 medium garlic cloves, finely chopped (1 Tbsp.)",
                "10 boneless, skinless chicken thighs (2 lb. 3 oz.), halved",
                "1⁄4 tsp. freshly ground black pepper",
                "1⁄4 cup fresh lemon juice",
                "1 cup plain whole-milk Greek yogurt",
                "1 large egg",
                "5 tbsp. unsalted butter, divided",
                "1 tbsp. unsalted butter",
                "1⁄2 cup dried barberries, picked, soaked in cold water for 15 minutes, then drained (or substitute finely chopped dried cranberries)",
                "1 tsp. sugar (omit if using cranberries)",
                "1⁄3 cup raw pistachios, coarsely chopped",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """instruction 0\nParboil the rice by following steps 1 and 2 of the Steamed Saffron Rice with Tahdig recipe (p. 36). Set the rice aside to drain as you prepare the dish’s other components.\ninstruction 1\nIn a small bowl, add the saffron and 2 tablespoons hot water. (Be sure to pre-soak your barberries now, too. See toppings list.)\ninstruction 2\nIn a large, deep skillet, add the olive oil and set over medium heat. Once hot, add the onion and ¼ teaspoon salt, and cook, stirring occasionally, until soft and golden, about 20 minutes. Add the garlic and cook until fragrant, 2 minutes. Add the chicken thighs, 2¾ teaspoons salt, and the pepper. Cook, turning the chicken as needed, until the pieces begin to brown slightly, about 6 minutes total. Add the lemon juice and saffron water, and turn the chicken to coat well. Lower the heat to medium-low, partially cover the pot, and let simmer, stirring occasionally, until the chicken is tender and just cooked through, 20–25 minutes. Remove from the heat and let cool slightly. Cut the chicken into ½-inch-long strips, return them to the pan, and toss to coat in the onions and pan juices. Remove the pan from the heat and set aside while you finish the rice.\ninstruction 3\nPreheat the oven to 400°F and set a rack in the lowest position.\ninstruction 4\nIn a large bowl, combine the yogurt, egg, and a tiny pinch of ground saffron. Add half of the rice and stir well to combine. Set the mixture aside.\ninstruction 5\nIn a 9-by-13-inch glass baking dish (preferably clear), add 3 tablespoons of butter; set the dish in the oven to melt the butter, about 3 minutes. Remove the dish, swirling the melted butter all over the bottom and sides. Spread the yogurt-rice mixture evenly along the bottom of the dish, packing it down firmly. Add the chicken pieces evenly over the rice, then sprinkle the remaining plain rice over the chicken in an even layer. Drizzle with 2 tablespoons of chicken juices and dot with the remaining 2 tablespoons of butter. Cover tightly with aluminum foil, transfer to the oven, and bake until the bottom of the rice is crispy and golden, 80–90 minutes. (If using a clear glass dish, carefully lift it to check the bottom.) Remove the baking dish and let the rice rest at room temperature for 5 minutes.\ninstruction 6\nWhile the tahcheen rests, prepare the topping: In a small pot, melt the butter over medium heat. Stir in the drained barberries, pistachios, and sugar (if using), and cook until the berries are just plumped, about 2 minutes. Remove from the heat.\ninstruction 7\nRun a knife along the sides of the baking dish to help release the rice. Place a large rectangular serving platter, baking tray, or cutting board over the tahcheen, take a deep breath, and quickly and confidently flip the baking dish over to unmold. You should hear a swish when the tahdig releases. Remove the baking dish and sprinkle the crispy surface of the rice with the barberry topping. Cut into pieces and serve.""",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.wholefoods import WholeFoods
from tests import ScraperTest


class TestWholeFoodsScraper(ScraperTest):

    scraper_class = WholeFoods

    def test_host(self):
        self.assertEqual("wholefoodsmarket.com", self.harvester_class.host())

    def test_host_domain(self):
        self.assertEqual(
            "wholefoodsmarket.co.uk", self.harvester_class.host(domain="co.uk")
        )

    def test_title(self):
        self.assertEqual("Grilled Cheese and Greens", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 tablespoons extra-virgin olive oil, divided",
                "1 clove garlic, thinly sliced",
                "1 bunch (about 8 ounces) kale, thick stems removed and leaves shredded",
                "8 slices whole grain bread",
                "8 slices Swiss or Le Gruyère cheese (about 8 ounces total)",
                "1/4 teaspoon kosher salt",
                "1/8 teaspoon ground black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """In a large skillet, heat 1 teaspoon olive oil over medium-high heat until hot.
Add kale, garlic, salt and pepper. Cover and cook, stirring occasionally, until kale begins to wilt, about 5 minutes.
Uncover, reduce heat to medium and continue to cook, stirring frequently, until kale is very soft, about 5 minutes.
Arrange 4 bread slices on a work surface. Top each with one slice of cheese, an even layer of the kale, another slice of cheese, and then a piece of bread.
Brush both sides of each sandwich with remaining olive oil.
Heat a large nonstick skillet over medium heat until hot.
Place stacked sandwiches in the pan and cook until bread is golden brown and cheese has melted, pressing flat with a spatula occasionally, 4 to 5 minutes per side. Serve immediately. Depending on the size of your skillet, you might need to cook these in batches. If so, the first batch can be kept warm in a 200°F oven while finishing the second batch.""",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "http://assets.wholefoodsmarket.com/recipes/3604/460/290/3604-1.jpg",
            self.harvester_class.image(),
        )



from responses import GET

from recipe_scrapers.woolworths import Woolworths
from tests import ScraperTest


class TestWoolworthsScraper(ScraperTest):

    scraper_class = Woolworths

    @classmethod
    def expected_requests(cls):
        yield GET, "https://www.woolworths.com.au/shop/recipes/asparagus-salad-with-lemon-vinaigrette", "tests/test_data/woolworths.testhtml"
        yield GET, "https://foodhub.woolworths.com.au/content/woolworths-foodhub/en/asparagus-salad-with-lemon-vinaigrette.model.json", "tests/test_data/woolworths.testhtml"

    def test_host(self):
        self.assertEqual("woolworths.com.au", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Fresh Ideas", self.harvester_class.author())

    def test_category(self):
        self.assertEqual("Salads", self.harvester_class.category())

    def test_title(self):
        self.assertEqual(
            "Asparagus Salad With Lemon Vinaigrette Recipe | Woolworths",
            self.harvester_class.title(),
        )

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_prep_time(self):
        self.assertEqual(10, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://foodhub.scene7.com/is/image/woolworthsltdprod/2010-asparagus-salad-with-lemon-vinaigrette:Square-1300x1300",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 bunches asparagus, trimmed, halved diagonally",
                "100g snow peas, trimmed, halved diagonally",
                "2 gem lettuce, trimmed, leaves separated",
                "2 radishes, trimmed, thinly sliced into rounds",
                "80g feta, crumbled",
                "0.5 cup small basil leaves",
                "2 tbs natural sliced almonds, toasted",
                "1 tbs white wine vinegar",
                "2 tsp Dijon mustard",
                "0.25 tsp sea salt flakes",
                "0.25 tsp caster sugar",
                "0.25 cup extra virgin olive oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Bring a medium saucepan of water to the boil over high heat. Cook asparagus and snow peas for 2 minutes or until just tender. Drain. Refresh under cold water. Drain.
Meanwhile, to make the vinaigrette, whisk all the ingredients together in a small jug until combined, then season.
Arrange lettuce, asparagus, snow peas and radish on a platter. Scatter over feta, drizzle with dressing, top with basil and almonds, then serve.""",
            self.harvester_class.instructions(),
        )

    def test_cuisine(self):
        self.assertEqual("French", self.harvester_class.cuisine())

    def test_cook_time(self):
        self.assertEqual(5, self.harvester_class.cook_time())

    def test_nutrients(self):
        self.assertEqual({}, self.harvester_class.nutrients())

    def test_language(self):
        self.assertEqual("en-AU", self.harvester_class.language())

    def test_site_name(self):
        self.assertEqual(
            "Woolworths | Fresh Ideas For You", self.harvester_class.site_name()
        )



from recipe_scrapers.inspiralized import Inspiralized
from tests import ScraperTest


class TestInspiralizedScraper(ScraperTest):

    scraper_class = Inspiralized

    def test_host(self):
        self.assertEqual("inspiralized.com", self.harvester_class.host())

    def test_language(self):
        self.assertEqual("en-US", self.harvester_class.language())

    def test_canonical_url(self):
        self.assertEqual(
            "https://inspiralized.com/brussels-sprouts-and-apple-salad-with-parmesan/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Brussels Sprouts and Apple Salad with Parmesan",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Ali Maffucci")

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 tablespoons extra virgin olive oil",
                "3 tablespoons apple cider vinegar",
                "2.5 teaspoons honey",
                "salt and pepper",
                "4 cups shredded brussels sprouts",
                "1 medium apple (Blade D)",
                "1/4 cup chopped raw almonds (for extra flavor, toast these first)",
                "1/3 cup shaved Parmesan",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "In a large bowl, whisk together the olive oil, apple cider vinegar, honey, and season with salt and pepper. Add the brussels sprouts and apples and toss well. Let sit in the refrigerator for at least 15-20 minutes and then take out and fold in the almonds and half of the Parmesan cheese. Transfer to a serving bowl or plate and top with remaining Parmesan.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.rachlmansfield import RachlMansfield
from tests import ScraperTest


class TestRachlMansfieldScraper(ScraperTest):

    scraper_class = RachlMansfield

    def test_host(self):
        self.assertEqual("rachlmansfield.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Rachel", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Healthy Flourless Brownies (nut-free + gluten-free)",
        )

    def test_total_time(self):
        self.assertEqual(27, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("9 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://rachlmansfield.com/wp-content/uploads/2019/12/B6984237-A840-46A7-93FD-417FAA609A78-scaled-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup tahini*",
                "1/4 cup maple syrup",
                "1/2 cup coconut sugar",
                "2 pasture-raised eggs*",
                "1 teaspoon vanilla extract",
                "1/3 cup + 2 tablespoons cacao powder",
                "1 teaspoon baking powder",
                "1/2 cup Hu Kitchen Dark Chocolate Gems (code RACHL for free shipping)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Preheat oven to 350 degrees and line an 8×8 baking dish with parchment paper and grease well\nCream together the tahini, maple syrup, coconut sugar, eggs and vanilla\nMix in the cacao powder and baking powder until well combined (it will be thick!)\nFold in dark chocolate gems then add batter to baking dish\nBake in oven for 22-25 minutes (or until toothpick comes out clean when you poke the brownies)\nAllow the brownies to cool for a few minutes (this is key so they set!) then slice and enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.seriouseats import SeriousEats
from tests import ScraperTest


class TestSeriousEats(ScraperTest):

    scraper_class = SeriousEats

    def test_host(self):
        self.assertEqual("seriouseats.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.seriouseats.com/old-fashioned-flaky-pie-dough-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Stella Parks")

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Buttery, Flaky Pie Crust Recipe"
        )

    def test_total_time(self):
        self.assertEqual(200, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("20 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.seriouseats.com/thmb/lDOD3TgZPxywf6S8Ca8N00raHvo=/1125x1125/smart/filters:no_upscale()/__opt__aboutcom__coeus__resources__content_migration__serious_eats__seriouseats.com__images__2016__09__20160712-blind-bake-pie-crust-vicky-wasik-10-306bb1237a62413fa754cac8e53db637.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "8 ounces low-protein all-purpose flour, such as Gold Medal Blue Label (1 2/3 cups; 225g), plus more for dusting",
                "1/2 ounce sugar (1 tablespoon; 15g)",
                "1 teaspoon (4g) Diamond Crystal kosher salt; for table salt, use half as much by volume or use the same weight",
                "8 ounces unsalted, American-style butter, straight from the fridge (2 sticks; 225g), cold",
                "4 ounces cold tap water (1/2 cup; 115g)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "For the Dough: Whisk flour, sugar, and salt together in a medium bowl. Cut butter into cubes no smaller than 1/2 inch, and toss with flour mixture to break up the pieces. With your fingertips, smash each cube flat—that's it! No rubbing or cutting. Stir in water, then knead dough against sides of the bowl until it comes together in a shaggy ball. Dough temperature should register between 65 and 70°F (18 and 21°C); if not, refrigerate briefly before rolling and folding (see note).\nMake the Layers: On a generously floured work surface, roll dough into a roughly 10- by 15-inch rectangle. Fold the 10-inch sides to the center, then close the newly formed packet like a book. Fold in half once more, bringing the short sides together to create a thick block. Divide in half with a sharp knife or bench scraper. Dough temperature should still be somewhere between 65 and 70°F (18 and 21°C); if not, refrigerate briefly before proceeding (see note).\nFor Single-Crusted Pies: Using as much flour as needed, roll one piece into a 14-inch circle; this size allows ample room to line pie plate, with enough overhang to form a generous border. At smaller sizes, dough will fall short, making it difficult to shape edges, and thicker dough will not crisp as intended. Transfer to 9-inch pie plate; dough should be easy to handle, and will not require any special procedures to move. Dust off excess flour with a pastry brush, using it to nestle dough into corners of pan. With scissors or kitchen shears, trim edge so that it overhangs by 1 1/4 inches. Fold overhang over itself to create thick border that sits on top edge of pie plate, not below. Crimp or shape crust as desired. Repeat with remaining dough. Wrap with plastic and refrigerate at least 2 hours and up to overnight. Use as directed in your favorite recipe.\nFor a Double-Crusted Pie: Using as much flour as needed, roll one piece into a 14-inch circle; this size allows ample room to line pie plate, with enough overhang to form a generous border. At smaller sizes, dough will fall short, making it difficult to shape edges, and thicker dough will not crisp as intended. Transfer to 9-inch pie plate; dough should be easy to handle, and will not require any special procedures to move. Dust off excess flour with a pastry brush, using it to nestle dough into corners of pan. With scissors or kitchen shears, trim edge so that it overhangs by 1 1/4 inches. For solid top crust, roll remaining dough as before; for lattice-top pie, roll into a 9- by 15-inch rectangle instead. Transfer to a baking sheet or parchment-lined cutting board. (The parchment will prevent dough from absorbing any savory odors from the board.) Wrap both portions in plastic and refrigerate at least 2 hours and up to overnight. Use as directed in your favorite recipe; after filling pie and sealing crusts together, refrigerate 30 minutes before baking.\nFor a Blind-Baked Pie : Adjust oven rack to lower-middle position and preheat to 350°F (180°C). Line pie shell that has been chilled for at least 2 hours (as outlined in Step 3) with large sheet of aluminum foil, pressing so it conforms to curves of plate. (A second sheet of foil may be needed for full coverage.) Fill to brim with sugar, transfer to a half sheet pan, and bake until fully set and golden around the edges, 60 to 75 minutes. Fold long sides of foil toward middle, gather short sides, and use both hands to carefully transfer sugar to heat-safe bowl. Let sugar cool to room temperature. If needed, continue baking crust a few minutes more to brown along the bottom.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.thevintagemixer import TheVintageMixer
from tests import ScraperTest


class TestTheVintageMixerScraper(ScraperTest):

    scraper_class = TheVintageMixer

    def test_host(self):
        self.assertEqual("thevintagemixer.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thevintagemixer.com/cherry-baby-birthday-cake-party/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Gluten Free and Sugar Free Cherry Baby Smash Cake",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Becky")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_image(self):
        self.assertEqual(
            "https://d6h7vs5ykbiug.cloudfront.net/wp-content/uploads/2017/08/Cherry-Baby-Birthday-Party-2-1-150x150.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 tablespoons coconut flour, +1 teaspoon",
                "1/4 cup almond flour",
                "1/4 teaspoon baking powder",
                "1/6 teaspoon baking soda",
                "1/4 teaspoon salt",
                "1 ripe banana, about 1/2 cup",
                "2 tablespoons coconut oil, room temp",
                "2 tablespoons almond butter, room temp",
                "1 large egg, beaten",
                "1 tablespoon pure maple syrup",
                "1/2 teaspoon pure vanilla extract",
                "1/2 cup cherries, pitted and chopped",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 375 and grease two small ramekins* with coconut oil.\nIn a small mixing bowl combine the dry ingredients: coconut flour, almond flour, baking powder, baking soda, and salt.\nIn a separate, medium-sized, bowl mash the banana then add in the coconut oil, almond butter, maple syrup and vanilla. Stir in the egg.\nCombine the dry ingredients to the wet and mix only until combined and smooth.\nToss the chopped cherries with a teaspoon or so of coconut flour. Stir these into the batter. Spoon batter out into ramekins and bake at 375 for 20-25 minutes.\nLet cool 5 minutes in the pan then remove to a wire rack to cool completely. Wrap in plastic and freeze.\nAbout 1 hour before serving, remove cakes from freezer. If the cakes are puffed up at the top slice off the top to even them out and make it flat to layer the cakes.\nDap a small amount of frosting under the first cake on the plate so it won't wiggle as you ice it then add a spoonful on top of the cake to place the second layer on top. Frost gently around the sides of the cake and on top. Top the cake with a few fresh cherries.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._exceptions import SchemaOrgException
from ._utils import normalize_string


class Rezeptwelt(AbstractScraper):
    @classmethod
    def host(cls):
        return "rezeptwelt.de"

    def author(self):
        return normalize_string(self.soup.find("span", {"id": "viewRecipeAuthor"}).text)

    def title(self):
        return self.soup.find("meta", {"property": "og:title"})["content"]

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        content = self.soup.find("ol", {"itemprop": "recipeInstructions"}).findAll(
            "div", {"itemprop": "itemListElement"}
        )
        res = ""
        for i in content:
            steps = i.findAll("span", {"itemprop": "text"})
            for step in steps:
                res += normalize_string(step.text) + "\n"
        return res

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        try:
            return self.schema.cuisine()
        except SchemaOrgException:
            return None

    def description(self):
        return self.schema.description().replace(
            " Mehr Thermomix ® Rezepte auf www.rezeptwelt.de", ""
        )

    def language(self):
        return self.soup.find("meta", {"property": "og:locale"})["content"]



# mypy: disallow_untyped_defs=False
import requests

from ._abstract import HEADERS, AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string, url_path_to_dict


class GoustoJson(AbstractScraper):
    """
    Ad-hoc solution to https://github.com/hhursev/recipe-scrapers/issues/376
    Let's see if it stands the test of time and reevaluate.
    """

    def __init__(self, url, proxies=None, timeout=None, *args, **kwargs):
        super().__init__(url=url, *args, **kwargs)

        recipe_slug = url_path_to_dict(url).get("path").split("/")[-1]
        data_url = (
            f"https://production-api.gousto.co.uk/cmsreadbroker/v1/recipe/{recipe_slug}"
        )

        recipe_json = requests.get(
            data_url,
            headers=HEADERS,
            proxies=proxies,
            timeout=timeout,
        ).json()

        self.page_data = recipe_json.get("data")
        self.data = self.page_data.get("entry")

    @classmethod
    def host(cls):
        return "gousto.co.uk"

    def title(self):
        return self.data.get("title")

    def total_time(self):
        return get_minutes(sorted(self.data.get("prep_times").values())[-1])

    def yields(self):
        return get_yields(sorted(self.data.get("prep_times").keys())[-1])

    def image(self):
        return self.data.get("seo").get("open_graph_image")

    def ingredients(self):
        return [
            normalize_string(ingredient.get("label"))
            for ingredient in self.data.get("ingredients")
            if isinstance(ingredient, dict) and "label" in ingredient.keys()
        ]

    def instructions(self):
        return "\n".join(
            [
                normalize_string(instruction.get("instruction"))
                for instruction in self.data.get("cooking_instructions")
                if isinstance(instruction, dict) and "instruction" in instruction.keys()
            ]
        )

    def ratings(self):
        return self.data.get("rating").get("average")
